#region Copyright (c) 2003-2004 ComponentGo!
/*
*********************************************************************
*                                                                   *
*       ComponentGo! Component Library                              *
*                                                                   *
*       Copyright (c) 2003-2004 ComponentGo!                        *
*       ALL RIGHTS RESERVED                                         *
*                                                                   *
*********************************************************************
*/
#endregion

using System;
using System.IO;
using System.Reflection;
using System.Drawing;
using System.Windows.Forms;

namespace ComponentGo.Calendars 
{

	#region Appointment

	/// <summary>
	/// Base class for a class of arguments based on <see cref="Appointment"/>.
	/// </summary>
	[System.Runtime.InteropServices.ComVisible(true)]
	public class AppointmentEventArgs : EventArgs 
	{

		/// <summary>
		/// Standard empty event.
		/// </summary>
		/// <remarks>
		/// It is used in cases, when no appointment is specified (<b>null</b> value).
		/// </remarks>
		public static readonly new AppointmentEventArgs Empty = 
			new AppointmentEventArgs(null);

		readonly Appointment ap;

		/// <summary>
		/// Constructs and initializes basic field of the object.
		/// </summary>
		/// <param name="appointment">An appointment being argument.</param>
		public AppointmentEventArgs(Appointment appointment) 
		{
			this.ap = appointment;
		}

		/// <summary>
		/// An <see cref="Appointment"/> being an argument of an event.
		/// </summary>
		public Appointment Appointment 
		{
			get { return ap; }
		}
	}

	/// <summary>
	/// Delegate invoked, when an event occurs.
	/// </summary>
	/// <param name="sender">The source of the event.</param>
	/// <param name="args">A <see cref="AppointmentEventArgs"/> that contains the event data.</param>
	[System.Runtime.InteropServices.ComVisible(true)]
	public delegate void AppointmentEventHandler(object sender, AppointmentEventArgs args);

	#endregion

	#region PaintAppointment

	/// <summary>
	/// Provides data for the <see cref="RootCalendar.PaintAppointment"/> event.
	/// </summary>
	[System.Runtime.InteropServices.ComVisible(true)]
	public class PaintAppointmentEventArgs : AppointmentEventArgs 
	{

		readonly Graphics g;
		bool userPaint = false;

		/// <summary>
		/// Constructs and initializes basic field of the object.
		/// </summary>
		/// <param name="g">The <see cref="Graphics"/> used to paint the appointment.</param>
		/// <param name="ap">An appointment being painted.</param>
		/// <remarks>
		/// Initializes <see cref="UserPaint"/> to <c>false</c>.
		/// </remarks>
		public PaintAppointmentEventArgs(Graphics g, Appointment ap) : base(ap) 
		{
			this.g = g;
		}

		/// <summary>
		/// Gets the graphics used to paint.
		/// </summary>
		public Graphics Graphics 
		{
			get { return g; }
		}

		/// <summary>
		/// Determines, whether an object was completely painted by a user.
		/// </summary>
		/// <remarks>
		/// Determines, whether an object was completely painted by a user.
		/// The standard Paint method is not invoked in this case.<para/>
		/// By default a propery is set to <c>false</c>. Therefore, after
		/// providing your own Paint method, you should set it to <c>true</c>.
		/// </remarks>
		public bool UserPaint 
		{
			get { return userPaint; }
			set { userPaint = value; }
		}

	}

	/// <summary>
	/// Delegate invoked, when it is required to repaint a calendar.
	/// <seealso cref="RootCalendar.PaintAppointment"/>
	/// <seealso cref="PaintAppointmentEventArgs"/>
	/// </summary>
	/// <param name="sender">The source of the event.</param>
	/// <param name="args">A <see cref="PaintAppointmentEventArgs"/> that contains the event data.</param>
	/// <remarks>
	/// When you delegate this class, you supply your own method to paint an appointment.
	/// This method can use a standard implementation of calendar's Paint method <see cref="DailyCalendar.Paint"/>.
	///  <para/>
	/// If you painted a whole appointment, and do not want a standard method to be invoked
	/// you should set a property
	/// <see cref="PaintAppointmentEventArgs.UserPaint"/> to <c>true</c>. Otherwise
	/// a calendar will invoke its method painting a calendar.
	/// </remarks>
	/// <example>
	/// An example shows how to write an appointment's Paint delegate
	/// that would block a standard
	/// method of painting a day, and invoke it itself.
	/// <code>
	/// private void cal_PaintAppointment(object sender, ComponentGo.Calendars.PaintAppointmentEventArgs args) {
	///		args.UserPaint = false;
	///		cal.Paint(args.Graphics, args.Appointment);
	///	}
	///	</code>
	/// </example>
	[System.Runtime.InteropServices.ComVisible(true)]
	public delegate void PaintAppointmentEventHandler(object sender, PaintAppointmentEventArgs args);

	#endregion

	#region EditAppointment

	/// <summary>
	/// Provides data for the <see cref="BaseCalendar.EditAppointment"/> event.
	/// </summary>
	[System.Runtime.InteropServices.ComVisible(true)]
	public class EditAppointmentEventArgs : AppointmentEventArgs 
	{

		Control control = null;
		bool cancel = false;
		bool continueEdit = false;

		/// <summary>
		/// Contructs and sets basic fields of an object.
		/// </summary>
		/// <param name="ap">The appoinments that begins to be edited.</param>
		public EditAppointmentEventArgs(Appointment ap) : base(ap) 
		{
		}

		/// <summary>
		/// A control performing an edit.
		/// </summary>
		/// <remarks>
		/// By default a property is set to <c>null</c>. 
		/// A standard <see cref="TextBox"/> class is then used to edit an appointment.<para/>
		/// If you want to substitute your own control to edit an appointment, you should set
		/// this property to your control. 
		/// </remarks>
		public Control Control 
		{
			get { return control; }
			set { control = value; }
		}

		/// <summary>
		/// Edit terminated with an ESC key.
		/// </summary>
		/// <remarks>
		/// Differentiates a situation, when an ESC key was pressed(<c>true</c>).<para/>
		/// Otherwise an edit is correctly finished (<c>false</c>)
		/// ex. by pressing an ENTER keyor by clicking a mouse outside the editor's window.
		/// </remarks>
		public bool Cancel 
		{
			get { return cancel; }
			set { cancel = value; }
		}

		/// <summary>
		/// Determines whether an operation should be continued.
		/// </summary>
		/// <remarks>
		/// Property is initially set to <c>false</c>. Setting it to <c>true</c> 
		/// makes it impossible to interrupt an edit of a meeting 
		/// ex when data entered by an user are incorrect.
		/// </remarks>
		public bool ContinueEdit 
		{
			get { return continueEdit; }
			set { continueEdit = value; }
		}
	}

	/// <summary>
	/// Delegate invoked, when an edit of the appointment starts or finishes.
	/// </summary>
	/// <seealso cref="RootCalendar.Edited"/>
	/// <seealso cref="RootCalendar.Editing"/>
	/// <seealso cref="EditAppointmentEventArgs"/>
	/// <param name="sender">The source of the event.</param>
	/// <param name="args">A <see cref="EditAppointmentEventArgs"/> that contains the event data.</param>
	[System.Runtime.InteropServices.ComVisible(true)]
	public delegate void EditAppointmentEventHandler(object sender, EditAppointmentEventArgs args);

	#endregion

	#region SplitAppointment

	/// <summary>
	/// Provides data for the <see cref="RootCalendar.SplitAppointment"/> event.
	/// </summary>
	[System.Runtime.InteropServices.ComVisible(true)]
	public class SplitAppointmentEventArgs : AppointmentEventArgs 
	{

		bool handled = false;
		DateTime splitTime;
		Appointment dragAp = null;

		/// <summary>
		/// Contructs and sets basic fields of an object.
		/// </summary>
		/// <param name="ap">The appoinment that begins to be splitted.</param>
		/// <param name="splitTime">Determines a time when a split operation should be performed.</param>
		public SplitAppointmentEventArgs(Appointment ap, DateTime splitTime) : base(ap) 
		{
			this.splitTime = splitTime;
		}

		/// <summary>
		/// Determines whether to perform a default split operation of an appointment.
		/// </summary>
		/// <remarks>
		/// An event can disallow a standard split operation is case a given appointment 
		/// cannot be split or split is handled within the event. 
		/// In these cases a value of this property should be set to <b>true</b>.
		/// </remarks>
		public bool Handled 
		{
			get { return handled; }
			set { handled = value; }
		}

		/// <summary>
		/// Determines a time when a split operation should be performed.
		/// </summary>
		/// <remarks>
		/// Time an appointment is to be split can be chand by setting this property to a new value. 
		/// If a <see cref="Handled"/> property is left <b>false</b>, an appointment will be split at a set time.
		/// </remarks>
		public DateTime SplitTime 
		{
			get { return splitTime; }
			set { splitTime = value; }
		}

		/// <summary>
		/// Determines an appointment, that will be dragged with a mouse after a split.
		/// </summary>
		public Appointment DragAppointment 
		{
			get { return dragAp; }
			set { dragAp = value; }
		}
	}

	/// <summary>
	/// Delegate invoked, when an appointment is split.
	/// </summary>
	/// <seealso cref="BaseCalendar.SplitMode"/>
	/// <seealso cref="RootCalendar.SplitAppointment"/>
	/// <seealso cref="SplitAppointmentEventArgs"/>
	/// <param name="sender">The source of the event.</param>
	/// <param name="args">A <see cref="SplitAppointmentEventArgs"/> that contains the event data.</param>
	[System.Runtime.InteropServices.ComVisible(true)]
	public delegate void SplitAppointmentEventHandler(object sender, SplitAppointmentEventArgs args);

	#endregion

	#region SplitAppointment

	/// <summary>
	/// Provides data for the <see cref="RootCalendar.MergeAppointments"/> event.
	/// </summary>
	[System.Runtime.InteropServices.ComVisible(true)]
	public class MergeAppointmentsEventArgs : AppointmentEventArgs 
	{

		bool cancel;
		readonly Appointment secondAp;

		/// <summary>
		/// Contructs and sets basic fields of an object.
		/// </summary>
		/// <param name="ap">The appoinment that begins to be merged.</param>
		/// <param name="secondAp">Determines the appointment, a current appointment is going to be 
		/// merged with.</param>
		/// <param name="cancel">Determines, whether merging is to be terminated.</param>
		public MergeAppointmentsEventArgs(Appointment ap, Appointment secondAp, bool cancel) : base(ap) 
		{
			this.secondAp = secondAp;
			this.cancel = cancel;
		}

		/// <summary>
		/// Determines, whether merging is to be terminated.
		/// </summary>
		/// <remarks>
		/// A value of <b>true</b> prevents appointment merging. 
		/// A value of <b>false</b> allows a calendar to merge two appointment objects.<para/>
		/// This property is initialised depending on whether merging a current appointment with 
		/// a neared linked one occurs. 
		/// If merging with a non-linked appointment occurs, this property is initialised to <b>true</b>.
		/// <seealso cref="Appointment.LinkWithNext"/>
		/// </remarks>
		public bool Cancel 
		{
			get { return cancel; }
			set { cancel = value; }
		}

		/// <summary>
		/// Determines the appointment, a current appointment is going to be merged with.
		/// </summary>
		/// <remarks>
		/// During merge, the second appointment will be deleted and the first appointment's <see cref="Appointment"/> 
		/// duration will be appropriately modified.
		/// </remarks>
		public Appointment SecondAppointment 
		{
			get { return secondAp; }
		}
	}

	/// <summary>
	/// Delegate invoked, when an appointments are merged.
	/// </summary>
	/// <seealso cref="RootCalendar.MergeAppointments"/>
	/// <seealso cref="MergeAppointmentsEventArgs"/>
	/// <param name="sender">The source of the event.</param>
	/// <param name="args">A <see cref="MergeAppointmentsEventArgs"/> that contains the event data.</param>
	[System.Runtime.InteropServices.ComVisible(true)]
	public delegate void MergeAppointmentsEventHandler(object sender, MergeAppointmentsEventArgs args);

	#endregion

	#region RequestAppointments

	/// <summary>
	/// Argument of an event notifying that data of appointment <see cref="RootCalendar.Appointments"/> 
	/// objects from some period have to be supplemented.
	/// </summary>
	/// <remarks>
	/// Properties of an event allowing to build an SQL statement (or determine a subset of data in other way) 
	/// reading a list of appointments, to be used by a control. 
	/// An algorithm allows to read partial data into a control. 
	/// There is no need to load all appointments from a data source.<para/>
	/// A condition determining whether a date <b>date</b> and resource <b>resource</b> are to be loaded: 
	/// <c>DateFrom&lt;=date &amp;&amp; date&lt;DateTo &amp;&amp; ResourceFrom&lt;=resource &amp;&amp; resource&lt;ResourceTo</c>.
	/// </remarks>
	[System.Runtime.InteropServices.ComVisible(true)]
	public class RequestAppointmentsEventArgs : EventArgs 
	{

		readonly DateTime dateFrom;
		readonly DateTime dateTo;
		readonly int resourceFrom;
		readonly int resourceTo;

		/// <summary>
		/// Contructs and sets basic fields of an object.
		/// </summary>
		/// <param name="dateFrom">Date and time the required period starts.</param>
		/// <param name="dateTo">Date and time the required period ends.</param>
		/// <param name="resourceFrom">The lowest resource index of a required data set.</param>
		/// <param name="resourceTo">The highest resource index of a required data set.</param>
		public RequestAppointmentsEventArgs(DateTime dateFrom, DateTime dateTo, int resourceFrom, int resourceTo) 
		{
			this.dateFrom = dateFrom;
			this.dateTo = dateTo;
			this.resourceFrom = resourceFrom;
			this.resourceTo = resourceTo;
		}

		/// <summary>
		/// Date and time the required period starts.
		/// </summary>
		/// <remarks>
		/// A condition determining, whether a date <b>value</b> is to be loaded: 
		/// <c>DateFrom&lt;=value &amp;&amp; value&lt;DateTo</c>.
		/// <seealso cref="DateTo"/>
		/// </remarks>
		public DateTime DateFrom 
		{
			get { return dateFrom; }
		}

		/// <summary>
		/// Date and time the required period ends.
		/// </summary>
		/// <remarks>
		/// A condition determining, whether a date <b>value</b> is to be loaded: <c>DateFrom&lt;=value &amp;&amp; value&lt;DateTo</c>.
		/// <seealso cref="DateFrom"/>
		/// </remarks>
		public DateTime DateTo 
		{
			get { return dateTo; }
		}

		/// <summary>
		/// The lowest resource index of a required data set.
		/// </summary>
		/// <remarks>
		/// A condition determining, whether a resource <b>value</b> is to be loaded: <c>ResourceFrom&lt;=value &amp;&amp; value&lt;ResourceTo</c>.
		/// <seealso cref="ResourceTo"/>
		/// </remarks>
		public int ResourceFrom 
		{
			get { return resourceFrom; }
		}

		/// <summary>
		/// The highest resource index of a required data set.
		/// </summary>
		/// <remarks>
		/// A condition determining, whether a resource <b>value</b> is to be loaded: <c>ResourceFrom&lt;=value &amp;&amp; value&lt;ResourceTo</c>.
		/// <seealso cref="ResourceFrom"/>
		/// </remarks>
		public int ResourceTo 
		{
			get { return resourceTo; }
		}

		/// <summary>
		/// Checks, whether an appointment satisfi1es condition set in a <see cref="RequestAppointmentsEventArgs"/> class.
		/// </summary>
		/// <remarks>
		/// An appointment object satisfying conditions of a class must: <list type="bullet"> <item><description>Okres czasu okrelony przez <see cref="Appointment.DateBegin"/> do <see cref="Appointment.DateEnd"/> musi przecina si (mie wsplny okres czasu)z okresem <see cref="DateFrom"/> do <see cref="DateTo"/>.</description></item> <item><description>Indekt kategorii <see cref="Appointment.ResourceIndex"/> musi zawiera sie w przedziale <see cref="ResourceFrom"/> do <see cref="ResourceTo"/>.
		///	</description></item>
		/// </list>
		/// </remarks>
		/// <param name="ap">An appointment object being checked.</param>
		/// <returns>Returns <b>true</b> if conditions of a class are satisfied. 
		/// Otherwise <b>false</b>.</returns>
		public bool Check(Appointment ap) 
		{
			return BaseCalendar.intersected(DateFrom, DateTo, ap) 
				&& ResourceFrom<=ap.ResourceIndex && ap.ResourceIndex<ResourceTo;
		}
	}

	/// <summary>
	/// Delegate invoked, when a calendar notifies, that a collection <see cref="RootCalendar.Appointments"/> 
	/// should be supplemented.
	/// </summary>
	/// <param name="sender">The source of the event.</param>
	/// <param name="args">A <see cref="RequestAppointmentsEventArgs"/> that contains the event data.</param>
	[System.Runtime.InteropServices.ComVisible(true)]
	public delegate void RequestAppointmentsEventHandler(object sender, RequestAppointmentsEventArgs args);

	#endregion

	#region QueryAppointmentDrag

	/// <summary>
	/// Argument of an event checking validity of a Drag&amp;Drop operation for a given appointment.
	/// </summary>
	/// <remarks>
	/// Some time and resource settings for Appointment objects are not allowed.
	/// Their validity is checked using a <see cref="BaseCalendar.QueryAppointmentDrag"/> event. 
	/// </remarks>
	[System.Runtime.InteropServices.ComVisible(true)]
	public class QueryAppointmentDragEventArgs : AppointmentEventArgs 
	{

		readonly bool isCopy;
		bool valid;

		/// <summary>
		/// Contructs and sets basic fields of an object.
		/// </summary>
		/// <param name="appointment">An appointment checked.</param>
		/// <param name="valid">A default value of a <see cref="Valid"/> property.</param>
		/// <param name="isCopy">Indicates that an operation of copying an appointment is in progress.</param>
		public QueryAppointmentDragEventArgs(Appointment appointment, bool valid, bool isCopy) : base(appointment) 
		{
			this.isCopy = isCopy;
			this.valid = valid;
		}

		/// <summary>
		/// Determines, whether a selected time period is acceptable and whether a Drag&amp;Drop operation 
		/// can be completed correctly.
		/// </summary>
		public bool Valid 
		{
			get { return valid; }
			set { valid = value; }
		}

		/// <summary>
		/// Indicates copying an Appointment using Drag&amp;Drop.
		/// </summary>
		public bool IsCopy 
		{
			get { return isCopy; }
		}
	}

	/// <summary>
	/// Delegate invoked, when a calendar notifies, that a collection should be supplemented.
	/// <see cref="RootCalendar.Appointments"/>.
	/// </summary>
	/// <param name="sender">The source of the event.</param>
	/// <param name="args">A <see cref="RequestAppointmentsEventArgs"/> that contains the event data.</param>
	[System.Runtime.InteropServices.ComVisible(true)]
	public delegate void QueryAppointmentDragEventHandler(object sender, QueryAppointmentDragEventArgs args);

	#endregion


	#region "StartAppointmentDragEventArgs"

	[System.Runtime.InteropServices.ComVisible(true)]
	public class StartAppointmentDragEventArgs : AppointmentEventArgs
	{
		public bool Cancel;
		public StartAppointmentDragEventArgs(Appointment app):base(app)
		{
			Cancel = false;
		}

	}

	[System.Runtime.InteropServices.ComVisible(true)]
	public delegate void StartAppointmentDragEventHandler(object sender, StartAppointmentDragEventArgs args);
	
	#endregion

	#region CalendarCreated

#if !DAILY

	/// <summary>
	/// AArguments of an event called when  new calendar 
	/// in a <see cref="ComboCalendar"/> control is created.
	/// </summary>
	/// <remarks>
	/// This event is a very good place to programmatically initialize newly created calendar.
	/// </remarks>
	[System.Runtime.InteropServices.ComVisible(true)]
	public class CalendarCreatedEventArgs : EventArgs 
	{

		readonly BaseCalendar calendarControl;
		readonly CalendarView view;

		/// <summary>
		/// Contructs and sets basic fields of an object.
		/// </summary>
		/// <param name="calendarControl">Newly created calendar control.</param>
		/// <param name="view">Current calendar view forcing creation of a new control.</param>
		public CalendarCreatedEventArgs(BaseCalendar calendarControl, CalendarView view) 
		{
			this.calendarControl = calendarControl;
			this.view = view;
		}

		/// <summary>
		/// Newly created calendar control.
		/// </summary>
		public BaseCalendar CalendarControl 
		{
			get { return calendarControl; }
		}

		/// <summary>
		/// Current calendar view forcing creation of a new control.
		/// </summary>
		public CalendarView View 
		{
			get { return view; }
		}
	}
		
	/// <summary>
	/// Delegate invoked, when a calendar notifies about creation 
	/// of a new calendar control through <see cref="ComboCalendar"/>. 
	/// This event is a good place to initialize a newly created control.
	/// </summary>
	[System.Runtime.InteropServices.ComVisible(true)]
	public delegate void CalendarCreatedEventHandler(object sender, CalendarCreatedEventArgs args);

#endif

	#endregion

}
