#region Copyright (c) 2003-2004 ComponentGo!
/*
*********************************************************************
*                                                                   *
*       ComponentGo! Component Library                              *
*                                                                   *
*       Copyright (c) 2003-2004 ComponentGo!                        *
*       ALL RIGHTS RESERVED                                         *
*                                                                   *
*********************************************************************
*/
#endregion

using System;
using System.Runtime.Serialization;
using System.Diagnostics;
using System.Collections;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Data;
using System.Windows.Forms;

namespace ComponentGo.Calendars {

#if !DAILY

	/// <summary>
	/// Monthly calendar control. Can be placed on a form.
	/// </summary>
	/// <remarks>
	/// A calendar displays days in a form of a table. 
	/// One week is contained by one row of this table. 
	/// It is possible to display many rows representing many weeks. 
	/// If a calendar contains many resources, one week is displayed 
	/// in many rows, for each resource separately.<para/>
	/// It is possible to combine Saturday and Sunday in a single column.
	/// </remarks>
	[Designer(typeof(Design.BaseCalendarDesigner), typeof(System.ComponentModel.Design.IDesigner))]
	[ToolboxItem(true)]
	[System.Runtime.InteropServices.ComVisible(true)]
	[System.Runtime.InteropServices.Guid("724C8078-7349-42b4-AF8B-BB2592F7164C")]
	public class MonthlyCalendar : TabularCalendar {
		
		#region General

		/// <summary>
		/// Constructor creating a calendar control. 
		/// Properties are initialized to standard values.
		/// </summary>
		public MonthlyCalendar() : this(null) {
		}

		internal MonthlyCalendar(RootCalendar root) : base(root, false, true) {
			SelectedAppointmentColor = SystemColors.Highlight;
		}

		internal override int DefaultVisibleWeeks {
			get { return 5; }
		}

		internal override ScrollBar Scroll {
			get { return vScroll; }
		}

		internal override void addAp(Appointment ap) {
			if (ap.ResourceIndex>=items.Length) return;

			int idxFrom = getDateIdx(ap.dateStart, false)/7;
			int idxTo = getDateIdx(ap.dateStop, true)/7;
			if (idxTo<0 || VisibleWeeks<=idxFrom) return;

			sortedItems.Add(ap);

			int iFrom = getDateIdx(ap.dateStart, false);
			int iTo = getDateIdx(ap.dateStop, true);

			HybridDictionary [] arr = items[ap.ResourceIndex];
			if (arr==null)
				items[ap.ResourceIndex] = arr = new HybridDictionary[VisibleWeeks];
			for (int i = Math.Max(0, idxFrom); i<=Math.Min(VisibleWeeks-1, idxTo); ++i) {
				HybridDictionary t = arr[i];
				if (t==null)
					arr[i] = t = new HybridDictionary();

				for (int idx = 0; ; ++idx) {
					bool ok = true;
					foreach (DictionaryEntry de in t) {
						if ((int)de.Value==idx) {
							Appointment a = (Appointment)de.Key;
							int jFrom = getDateIdx(a.dateStart, false);
							int jTo = getDateIdx(a.dateStop, true);
							if (jTo>=iFrom && iTo>=jFrom) {
								ok = false;
								break;
							}
						}
					}
					if (ok) {
						t[ap] = idx;
						break;
					}
				}
			}
		}

		#endregion

		#region ViewMode

		MonthlyViewModes viewMode = MonthlyViewModes.FirstMonday;

		/// <summary>
		/// Determines the first day on the left side of a calendar or placing 
		/// Saturday and Sunday in a single column.
		/// </summary>
		[Description("Determines the first day on the left side of a calendar or placing Saturday and Sunday in a single column.")]
		[Category("Appearance")]
		[DefaultValue(MonthlyViewModes.FirstMonday)]
		public MonthlyViewModes ViewMode {
			get { return viewMode; }
			set { 
				if (viewMode==value) return;
				bool prevSunday = isFirstSunday;
				viewMode = value;
				if (prevSunday && !isFirstSunday)
					FirstDate = FirstDate.AddDays(1);
				else if (!prevSunday && isFirstSunday)
					FirstDate = FirstDate.AddDays(-1);
				Invalidate();
			}
		}

		bool isFirstSunday { 
			get { return ViewMode==MonthlyViewModes.FirstSunday; }
		}

		internal override DateTime GetRoundedDate(DateTime date) {
			if (isFirstSunday)
				return CalendarTools.GetSundayDate(date);
			return CalendarTools.GetMondayDate(date);
		}

		int NCols {
			get { 
				if (ViewMode==MonthlyViewModes.SaturdayWithSunday && !isResourceBar)
					return 6;
				else
					return 7;
			}
		}

		#endregion

		#region AlternativeMonthColor

		Color alternativeMonthColor = Color.Khaki;

		/// <summary>
		/// Alternative color for workdays in a month.
		/// </summary>
		/// <remarks>
		/// Alternative color of workdays in a month used in previous 
		/// and next month and every second month.
		/// </remarks>
		[Category("Colors")]
		[Description("Alternatywny kolor dni roboczych w miesicu.")]
		public Color AlternativeMonthColor {
			get { return alternativeMonthColor; }
			set {
				if (alternativeMonthColor==value) return;
				alternativeMonthColor = value;
				Invalidate();
			}
		}

		/// <summary>
		/// Indicates whether the <see cref="AlternativeMonthColor"/> property should be persisted.
		/// </summary>
		/// <returns><b>true</b> if the property value has changed from its 
		/// default; otherwise, <b>false</b>.</returns>
		/// <remarks>You typically use this method if you are creating a designer
		/// for <see cref="MonthlyCalendar"/>.
		/// </remarks>
		public bool ShouldSerializeAlternativeMonthColor() {
			return alternativeMonthColor!=Color.Khaki;
		}

		#endregion

		#region Rectangles

		const int labelHeight = 20;
		const int dayLabelHeight = 20;

		int resourceWidth {
			get { return isResourceBar ? 60 : 0; }
		}

		int workspaceWidth {
			get { return Width-resourceWidth-VerticalScrollBarWidth; }
		}

		/// <summary>
		/// Calculates a rectangle covering the selected region of a control.
		/// </summary>
		/// <param name="region">Region, the rectangle is calculated for.</param>
		/// <returns>Rectangle calculated relative to a calendar control.</returns>
		public Rectangle GetRegionRectangle(Regions region) {
			switch (region) {
				case Regions.DaysLabel:
					return new Rectangle(
						resourceWidth, labelHeight,
						workspaceWidth, labelHeight);

				case Regions.DaysWorkspace:
					return new Rectangle(
						resourceWidth, labelHeight*2,
						workspaceWidth, Height-labelHeight*2);

				case Regions.ResourcesLabel:
					if (isResourceBar)
						return new Rectangle(
							0, 0, 
							resourceWidth, Height);
					return Rectangle.Empty;
			}
			throw new ArgumentException();
		}

		int GetColumnX(int column) {
			int ncol;
			if (ViewMode==MonthlyViewModes.SaturdayWithSunday)
				ncol = 6;
			else
				ncol = 7;
			int w = workspaceWidth;
			if (column>=ncol) {
				if (column==6 && isResourceBar)
					return w + resourceWidth - w/ncol/2;
				return w + resourceWidth;
			}
			return column*w/ncol + resourceWidth;
		}

		int GetRowY(int row) 
		{
			int h = Height-labelHeight;
			if (row>=VisibleWeeks)
				return Height;
			return row*h/VisibleWeeks + labelHeight*2;
		}

		/// <summary>
		/// Rectangle on a control, corresponding to one day on a calendar.
		/// </summary>
		/// <param name="date">A date in a calendar.</param>
		/// <returns>
		/// Rectangle calculated relative to a left upper corner of a calendar control.
		/// </returns>
		public Rectangle GetDayRectangle(DateTime date) {
			Rectangle rt = Rectangle.Empty;
		
			if (FirstDate<=date && date<=LastDate) {
				int i = getDateIdx(date, false);
				int week = i/7;
				int dow = Math.Min(i%7, NCols-1);
				
				int x = GetColumnX(dow);
				int y = GetRowY(week);
				rt = new Rectangle(x, y, GetColumnX(dow+1)-x, GetRowY(week+1)-y);
				
				if (ViewMode==MonthlyViewModes.SaturdayWithSunday && !isResourceBar && i%7>=5) {
					int half = rt.Height/2;
					if (i%7==5)
						rt.Height = half;
					else {
						rt.Y += half;
						rt.Height -= half;
					}
				}
			}

			return rt;
		}

		/// <summary>
		/// Rectangle on a control, corresponding to one resource in a day on a calendar.
		/// </summary>
		/// <param name="date">A date in a calendar.</param>
		/// <param name="resourceIndex">A resource index in a calendar.</param>
		/// <returns>
		/// Rectangle calculated relative to a left upper corner of a calendar control.
		/// </returns>
		public Rectangle GetResourceRectangle(DateTime date, int resourceIndex) {
			Rectangle rt = GetDayRectangle(date);
			if (rt==Rectangle.Empty) return rt;

			int y1 = resourceIndex * (rt.Height-dayLabelHeight) / ResCount;
			int y2 = (resourceIndex+1) * (rt.Height-dayLabelHeight) / ResCount;

			rt.Y += y1+dayLabelHeight;
			rt.Height = y2-y1;

			return rt;
		}

		/// <summary>
		/// Returns a rectangles on a screen containing a given appointment.
		/// </summary>
		/// <param name="ap">Appointment, the rectangles are returned for.</param>
		/// <returns>A table of rectangles, calculated relative to 
		/// an upper left corner of the calendar control. 
		///	</returns>
		public override Rectangle [] GetAppointmentRectangles(Appointment ap) {
			int iFrom = getDateIdx(ap.dateStart, false);
			int iTo = getDateIdx(ap.dateStop, true);
			int idxFrom = Math.Max(iFrom, 0);
			int idxTo = Math.Min(iTo, 7*VisibleWeeks);
			if (idxFrom>idxTo) return new Rectangle[0];

			int weekFrom = idxFrom/7;
			int weekTo = idxTo/7;
			int count = weekTo-weekFrom+1;
			if (ViewMode==MonthlyViewModes.SaturdayWithSunday) {
				count *= 2;
				if (iTo<VisibleWeeks*7 && iTo % 7!=6)
					--count;

			}
			Rectangle[] rects = new Rectangle[count];

			int rtidx = 0;
			for (int i = weekFrom; i<=weekTo; ++i) {
				HybridDictionary dic = GetItems(i, ap.ResourceIndex);
				object idx = dic[ap];
				if (idx==null)
					rects[rtidx++] = Rectangle.Empty;
				else {
					Rectangle rt = Rectangle.Empty;

					DateTime dateFrom = i==weekFrom && iFrom==idxFrom ?
						ap.dateStart.Date : FirstDate.AddDays(i*7);
					DateTime dateTo = i==weekTo && iTo==idxTo ?
						FirstDate.AddDays(idxTo) : FirstDate.AddDays((i+1)*7-1);

					bool withSunday = ViewMode==MonthlyViewModes.SaturdayWithSunday 
						&& dateTo.DayOfWeek==DayOfWeek.Sunday;
					if (withSunday)
						dateTo = dateTo.AddDays(-1);

					rt = GetResourceRectangle(dateFrom, ap.ResourceIndex);
					Rectangle rtTo = GetResourceRectangle(dateTo, ap.ResourceIndex);

					rt.Width = rtTo.Right-rt.Left;
				
					rects[rtidx++] = recalcAppRt(rt, idx);

					if (withSunday) {
						rt = GetResourceRectangle(dateTo.AddDays(1), ap.ResourceIndex);
						rects[rtidx++] = recalcAppRt(rt, idx);
					}
				}
			}

			return rects;
		}

		Rectangle recalcAppRt(Rectangle rt, object idx) {
			int maxY = rt.Bottom;
			rt.Y += appointmentHeight*(int)idx;
			rt.Height = appointmentHeight-2;
			rt.X += 2;
			rt.Width -= 6;
			if (rt.Bottom>maxY)
				rt = Rectangle.Empty;
			return rt;
		}

		#endregion

		#region Hits

		/// <summary>
		/// Calculates a calendar region located at given coordinates.
		/// </summary>
		/// <param name="pt">Coordinates relative to a calendar control
		/// the region is calculated for.</param>
		/// <returns>Calculated region of a calendar control.</returns>
		public Regions HitTest(Point pt) {
			if (GetRegionRectangle(Regions.DaysLabel).Contains(pt)) return Regions.DaysLabel;
			if (GetRegionRectangle(Regions.ResourcesLabel).Contains(pt)) return Regions.ResourcesLabel;
			return Regions.DaysWorkspace;
		}

		/// <summary>
		/// Calculates a day and an hour located at given coordinates.
		/// </summary>
		/// <param name="pt">
		/// Coordinates relative to a calendar control, an hour is calculated for.
		/// </param>
		/// <returns>Structure containing calculated information on day and hour.</returns>
		public override DateTime HitDateTimeTest(Point pt) {
			for (DateTime dt = LastDate; dt>=FirstDate; dt = dt.AddDays(-1))
				if (GetDayRectangle(dt).Contains(pt))
					return dt;

			return DateTime.MinValue;
		}

		/// <summary>
		/// Calculates an index of a resource located at a given point.
		/// </summary>
		/// <remarks>
		/// If there is no resource row at a given coordinates, -1 is returned.
		/// </remarks>
		/// <param name="pt">Coordinates relative to a calendar control, 
		///		an resource is calculated for.</param>
		/// <returns>An index of a resource.</returns>
		public override int HitResourceTest(Point pt) {
			for (int week = 0; week<VisibleWeeks; ++week) {
				for (int res = 0; res<ResCount; ++res) {
					Rectangle rt = GetResourceRectangle(FirstDate.AddDays(7*week), res);
					if (pt.Y<rt.Bottom) return res;
				}
			}
			return -1;
		}

		#endregion

		#region Paint

		#region Resources Label

		void paintResourcesLabel(Graphics g) {
			if (!isResourceBar) return;

			Rectangle rt = GetRegionRectangle(Regions.ResourcesLabel);
			if (rt.Width<=0) return;

			CalendarTools.FillRectangle(g, rt, ResourcesLabelColor, ResourcesLabelColorSecond, false);

			using (Pen linePen = new Pen(ControlPaint.LightLight(LineColor), 1))
			using (StringFormat textFormat = new StringFormat(StringFormatFlags.DisplayFormatControl | StringFormatFlags.NoWrap))
			using (Brush textBrush = new SolidBrush(ForeColor))
			using (Brush textSelBrush = new SolidBrush(SelectedAppointmentTextColor)) {
				CalendarTools.SetAlignment(textFormat, TextResourceAlign);
				textFormat.Trimming = StringTrimming.EllipsisWord;
					
				for (int week = 0; week<VisibleWeeks; ++week) {
					DateTime d1 = FirstDate.AddDays(7*week);
					DateTime d2 = d1.AddDays(7);
					bool isSelectedDays = intersected(d1, d2, SelectedDateBegin, SelectedDateEnd);

					for (int res = 0; res<ResCount; ++res) {
						Rectangle resRt = GetResourceRectangle(FirstDate.AddDays(7*week), res);
						resRt.X = rt.X;
						resRt.Width = rt.Width;

						if (res>0)
							g.DrawLine(linePen, 0, resRt.Y-1, Width, resRt.Y-1);

						bool isSelected = isSelectedDays && SelectedResource==res;
						if (isSelected)
							using (Brush selectedBrush = new SolidBrush(SelectedColor))
								g.FillRectangle(selectedBrush, resRt);
					
						resRt.Inflate(-1, -1);
						if (resRt.Width>0) {
							g.DrawString(Resources[res].Text, Font, 
								isSelected ? textSelBrush : textBrush, 
								resRt, textFormat);
						}
					}
				}
			}
		}

		#endregion

		#region Days Label

		void paintDaysLabel(Graphics g) {
			DateTime today = DateTime.Today;

			using (Pen linePen = new Pen(LineColor, 1))
			using (StringFormat textFormat = new StringFormat(StringFormatFlags.DisplayFormatControl | StringFormatFlags.NoWrap))
			using (Brush textBrush = new SolidBrush(ForeColor)) {
				CalendarTools.SetAlignment(textFormat, TextDayAlign);
				textFormat.Trimming = StringTrimming.EllipsisWord;

				Rectangle rt = GetRegionRectangle(Regions.DaysLabel);
				CalendarTools.FillRectangle(g, rt, DaysLabelColor, DaysLabelColorSecond, true);

				for (int col = 0; col<NCols; ++col) {
					DateTime date = FirstDate.AddDays(col);
					Rectangle dayRt = GetDayRectangle(date);

					if (col>0)
						g.DrawLine(linePen, dayRt.X-1, rt.Bottom, dayRt.X-1, Height);

					dayRt.Y = rt.Y;
					dayRt.Height = rt.Height;

					ControlPaint.DrawBorder3D(g, dayRt);
					dayRt.Inflate(-2, -2);
					if (dayRt.Width>0) {
						string s;
						if (col>=5 && ViewMode==MonthlyViewModes.SaturdayWithSunday) {
							if (isResourceBar)
								s = string.Format("{0:ddd}", date);
							else
								s = string.Format("{0:ddd}/{1:ddd}", date, date.AddDays(1));
						}
						else
							s = string.Format("{0:dddd}", date);
						g.DrawString(s, Font, textBrush, dayRt, textFormat);
					}
				}
			}
		}

		#endregion

		#region Days Workspace

		void paintDaysWorkspace(Graphics g) {
			DateTime today = DateTime.Today;
			Rectangle rect = GetRegionRectangle(Regions.DaysWorkspace);
			if (rect.Width<=0 || rect.Height<=0) return;

			CalendarTools.FillRectangle(g, rect, WorkHourColor, WorkHourColorSecond, true);

			using (StringFormat textFormat = new StringFormat(StringFormatFlags.DisplayFormatControl | StringFormatFlags.NoWrap))
			using (Brush textBrush = new SolidBrush(ForeColor))
			using (Brush textSelectedBrush = new SolidBrush(SelectedAppointmentTextColor))
			using (Pen linePen = new Pen(LineColor, 1)) {
				CalendarTools.SetAlignment(textFormat, ContentAlignment.MiddleRight);

				for (int week = 0; week<VisibleWeeks; ++week) {
					DateTime dateWeek = FirstDate.AddDays(7*week);
					for (int day = 0; day<7; ++day) {
						DateTime date = dateWeek.AddDays(day);
						Rectangle dayRt = GetDayRectangle(date);

						GetDayInfoEventArgs gdi = new GetDayInfoEventArgs(this, date);
						OnGetDayInfo(gdi);

						Color dayColor = Color.Empty;
						if (gdi.IndividualWorkColor || gdi.FreeDay)
							dayColor = gdi.FreeDay ? gdi.FreeHourColor : gdi.WorkHourColor;
						if ((today.Month+date.Month)%2!=0 && dayColor==Color.Empty)
							dayColor = AlternativeMonthColor;

						if (dayColor!=Color.Empty)
							using (Brush br = new SolidBrush(dayColor))
								g.FillRectangle(br, dayRt);

						dayRt.Height = dayLabelHeight;

						g.DrawLine(linePen, dayRt.Left, dayRt.Top-1, dayRt.Right, dayRt.Top-1);
	
						////////////////////////////////////////////////////
						// Label
						//
						--dayRt.Width;
						dayRt.Inflate(-1, -1);
						bool isSelected = SelectedDateBegin<=date && date<SelectedDateEnd;
						bool isToday = TodayMarkerVisible && date==today;
						if (isSelected || isToday) {

							CalendarTools.FillRectangle(g, dayRt, 
								isSelected ? SelectedColor : Color.FromArgb(0, 0, 0, 0), 
								isToday ? ControlPaint.LightLight(TodayMarkerColor) : SelectedColor, true);

							if (isToday)
								using (Pen todayPen = new Pen(TodayMarkerColor))
									g.DrawLine(todayPen, dayRt.Left, dayRt.Bottom-2, dayRt.Right, dayRt.Bottom-2);
						}

						string f;
						if (date.Day==1 || week==0 && day==0)
							f = "{0:MMMM yyyy} {0:%d}";
						else
							f = "{0:%d}";
						dayRt.Inflate(-1, -1);
						g.DrawString(string.Format(f, date), Font, 
							isSelected ? textSelectedBrush : textBrush, 
							dayRt, textFormat);
					}
				}
			}
		}

		#endregion

		/// <summary>
		/// Overloaded. Implementation of a calendar.
		/// </summary>
		/// <param name="pe">Like overloaded</param>
		protected override void OnPaint(PaintEventArgs pe) {
			Graphics g = pe.Graphics;
			initMargin(g);

			paintDaysWorkspace(g);
			paintResourcesLabel(g);
			paintDaysLabel(g);

			GetItems(0, 0);

			foreach (Appointment ap in Appointments)
				paintAppointment(g, ap);
			base.OnPaint(pe);
		}

		#endregion

		#region Keyboard

		internal override bool processKey(KeyEventArgs e) {
			int selRes;
			TimeSpan ts;
			switch (e.KeyData) {
				case Keys.Down:					
				case Keys.Shift | Keys.Down:					
					if (addColumnIndex(1, out selRes, out ts))
						moveCursor(e.Shift, ts, OneDay, selRes); 
					return true;
				case Keys.Up:					
				case Keys.Shift | Keys.Up:
					if (addColumnIndex(-1, out selRes, out ts))
						moveCursor(e.Shift, -ts, OneDay, selRes); 
					return true;
				case Keys.Right:				
				case Keys.Shift | Keys.Right:
					moveCursor(e.Shift, OneDay, OneDay, SelectedResource); 
					return true;
				case Keys.Left:					
				case Keys.Shift | Keys.Left:					
					moveCursor(e.Shift, -OneDay, OneDay, SelectedResource); 
					return true;
				case Keys.PageDown:				
				case Keys.Shift | Keys.PageDown:
					moveCursor(e.Shift, TimeSpan.FromDays(7*VisibleWeeks), OneDay, SelectedResource); 
					return true;
				case Keys.PageUp:
				case Keys.Shift | Keys.PageUp:
					moveCursor(e.Shift, -TimeSpan.FromDays(7*VisibleWeeks), OneDay, SelectedResource); 
					return true;
				case Keys.Home:					
				case Keys.Shift | Keys.Home:					
				case Keys.Control | Keys.Home:	
				case Keys.Shift | Keys.Control | Keys.Home:	
					moveCursor(e.Shift, CalendarTools.GetMondayDate(SelectedDateBegin), OneDay, SelectedResource); 
					return true;
				case Keys.End:					
				case Keys.Shift | Keys.End:					
				case Keys.Control | Keys.End:	
				case Keys.Shift | Keys.Control | Keys.End:	
					moveCursor(e.Shift, CalendarTools.GetMondayDate(SelectedDateBegin).AddDays(6), OneDay, SelectedResource); 
					return true;
			}
			return false;
		}

		bool addColumnIndex(int delta, out int resourceIndex, out TimeSpan ts) {
			resourceIndex = SelectedResource;
			ts = new TimeSpan(7, 0, 0, 0);

			if (!isResourceBar) {
				if (ViewMode==MonthlyViewModes.SaturdayWithSunday) {
					switch (SelectedDateBegin.DayOfWeek) {
						case DayOfWeek.Saturday:
							if (delta>0)
								ts = new TimeSpan(1, 0, 0, 0);
							else
								ts = new TimeSpan(6, 0, 0, 0);
							break;
						case DayOfWeek.Sunday:
							if (delta>0)
								ts = new TimeSpan(6, 0, 0, 0);
							else
								ts = new TimeSpan(1, 0, 0, 0);
							break;
					}
				}
				return true;
			}

			resourceIndex += delta;
			if (resourceIndex>=Resources.Count) {
				resourceIndex = 0;
				return true;
			}
			else if (resourceIndex<0) {
				resourceIndex = Resources.Count-1;
				return true;
			}
			SelectedResource = resourceIndex;
			return false;
		}

		#endregion
	}

#endif
}
