#region Copyright (c) 2003-2004 ComponentGo!
/*
*********************************************************************
*                                                                   *
*       ComponentGo! Component Library                              *
*                                                                   *
*       Copyright (c) 2003-2004 ComponentGo!                        *
*       ALL RIGHTS RESERVED                                         *
*                                                                   *
*********************************************************************
*/
#endregion

using System;
using System.Runtime.Serialization;
using System.Diagnostics;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

namespace ComponentGo.Calendars {

	/// <summary>
	/// A base class for all calendar controls.
	/// </summary>
	/// <remarks>
	/// Delivers funtionality common for these controls and their common properties.
	/// </remarks>
	[System.Runtime.InteropServices.ComVisible(true)]
	public abstract class RootCalendar : System.Windows.Forms.Control 
	{

		#region General
		
		/// <summary>
		/// Construct calendar object.
		/// </summary>
		/// <remarks>
		/// Initializes all fields of an object a constructs <see cref="Appointments"/> and <see cref="Resources"/> collections.
		/// </remarks>
		/// <param name="root">A managing calendar containing a calendar being created.</param>
		protected RootCalendar(RootCalendar root) 
		{
			AllowDrop = true;
			if (root==null) 
			{
				this.appointments = new AppointmentCollection();
				this.resources = new ResourceCollection();
			}
			else 
			{
				this.appointments = root.Appointments;
				this.resources = root.Resources;
			}
		}

		internal bool shouldSerialize() 
		{
			return !(Parent is RootCalendar);
		}

		#endregion

		#region Overrides

		/// <summary>
		/// Overrided. Implementation of a calendar control.
		/// </summary>
		[Browsable(false)]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public new string Text 
		{
			get { return base.Text; }
		}

		/// <summary>
		/// Overrided. Implementation of a calendar control.
		/// </summary>
		protected override void OnCreateControl() 
		{
			base.OnCreateControl();
			//Forms.Demo.Show(true);
		}

		#endregion

		#region Collections




		readonly AppointmentCollection appointments;

		/// <summary>
		/// A list of appointments displayed in a calendar.
		/// </summary>
		/// <remarks>
		/// Collection contains a set of objects of type <see cref="Appointment"/>. 
		/// This collection's methods can be used to add new and delete 
		/// existing appointmets. <para/>
		/// Collection should be populated before the control is opened. 
		/// It can also be successively populated when the control requires 
		/// it using the RequestAppointments event. 
		/// </remarks>
		[Category("Data")]
		[Description("A list of appointments displayed in a calendar")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
		public AppointmentCollection Appointments 
		{
			get { return appointments; }
		}

		/// <summary>
		/// Indicates whether the <see cref="Appointments"/> property should be persisted.
		/// </summary>
		/// <remarks>
		/// You typically use this method if you are creating a designer for <see cref="RootCalendar"/>.
		/// </remarks>
		/// <returns><b>true</b> if the property value has changed from its default; 
		/// otherwise, <b>false</b>.</returns>
		public bool ShouldSerializeAppointments() 
		{
			return shouldSerialize();
		}


		readonly ResourceCollection resources;

		/// <summary>
		/// A list of resources displayed in a control.
		/// </summary>
		/// <remarks>
		/// Collection contain objects of a  <see cref="Resource"/> class, 
		/// describing resources grouping Appointment object. 
		/// Methods of a <see cref="ResourceCollection"/> class allow for 
		/// addition and deletion of calendar's resources.
		/// </remarks>
		[Category("Data")]
		[Description("A list of resources displayed in a control.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
		public ResourceCollection Resources {
			get { return resources; }
		}

		/// <summary>
		/// Indicates whether the <see cref="Resources"/> property should be persisted.
		/// </summary>
		/// <remarks>
		/// You typically use this method if you are creating a designer for <see cref="RootCalendar"/>.
		/// </remarks>
		/// <returns><b>true</b> if the property value has changed from its default; 
		/// otherwise, <b>false</b>.</returns>
		public bool ShouldSerializeResources() {
			return shouldSerialize();
		}

		#endregion

		#region Behavior


		/// <summary>
		/// A property is redeclared to change its <see cref="DefaultValueAttribute"/>.
		/// </summary>
		[DefaultValue(true)]
		public new bool AllowDrop {
			get { return base.AllowDrop; }
			set { base.AllowDrop = value; }
		}

		/// <summary>
		/// Indicates whether the <see cref="AllowDrop"/> property should be persisted.
		/// </summary>
		/// <remarks>
		/// You typically use this method if you are creating a designer for <see cref="RootCalendar"/>.
		/// </remarks>
		/// <returns><b>true</b> if the property value has changed from its default; 
		/// otherwise, <b>false</b>.</returns>
		public bool ShouldSerializeAllowDrop() {
			RootCalendar root = Parent as RootCalendar;
			return root==null || root.AllowDrop!=AllowDrop;
		}

		bool allowOverlap = true;

		/// <summary>
		/// Allows to overlay appointments during Drag&amp;Drop.
		/// </summary>
		/// <remarks>
		/// Setting a value to <b>false</b> disallows placing more than one appointment 
		/// at at the same time in a same resource. 
		/// Many appointment can be placed at the same time, but in different resources.
		/// </remarks>
		[Category("Behavior")]
		[Description("Allows to overlay appointments during Drag&Drop.")]
		[DefaultValue(true)]
		public bool AllowOverlap {
			get { return allowOverlap; }
			set { allowOverlap = value; }
		}

		/// <summary>
		/// Indicates whether the <see cref="AllowOverlap"/> property should be persisted.
		/// </summary>
		/// <remarks>
		/// You typically use this method if you are creating a designer for <see cref="RootCalendar"/>.
		/// </remarks>
		/// <returns><b>true</b> if the property value has changed from its default; 
		/// otherwise, <b>false</b>.</returns>
		public bool ShouldSerializeAllowOverlap() {
			RootCalendar root = Parent as RootCalendar;
			return root==null || root.AllowOverlap!=AllowOverlap;
		}

		GrowMode growMode = GrowMode.ChangeLinkTime;

		/// <summary> 
		/// Determines how a date/time of a linked appointment located at a point of contact 
		/// of two linked appointmets is changed. 
		/// </summary> 
		/// <remarks> 
		/// This setting is used, when a program operator modifies using a mouse, 
		/// an end date of an appointment linked to the next one. 
		/// It makes it possible to select a calendar's behaviour, 
		/// to facilitate ex. hotel reservations or project management. 
		/// </remarks> 
		[Category("Behavior")]
		[Description("Determines how a date/time of a linked appointment located at a point of contact of two linked appointmets is changed.")]
		[DefaultValue(GrowMode.ChangeLinkTime)]
		public GrowMode GrowMode {
			get { return growMode; }
			set { growMode = value; }
		}

		/// <summary>
		/// Indicates whether the <see cref="GrowMode"/> property should be persisted.
		/// </summary>
		/// <remarks>
		/// You typically use this method if you are creating a designer for <see cref="RootCalendar"/>.
		/// </remarks>
		/// <returns><b>true</b> if the property value has changed from its default; 
		/// otherwise, <b>false</b>.</returns>
		public bool ShouldSerializeGrowMode() {
			RootCalendar root = Parent as RootCalendar;
			return root==null || root.GrowMode!=GrowMode;
		}

		bool enableEditInPlace = true;
		
		/// <summary>
		/// Value of 'true' enables editing an appointment text in a control.
		/// <seealso cref="Appointment.Edit"/>
		/// </summary>
		[Category("Behavior")]
		[Description("Value of 'true' enables editing an appointment text in a control")]
		[DefaultValue(true)]
		public bool EnableEditInPlace {
			get { return enableEditInPlace; }
			set { enableEditInPlace = value; }
		}

		/// <summary>
		/// Indicates whether the <see cref="EnableEditInPlace"/> property should be persisted.
		/// </summary>
		/// <remarks>
		/// You typically use this method if you are creating a designer for <see cref="RootCalendar"/>.
		/// </remarks>
		/// <returns><b>true</b> if the property value has changed from its default; 
		/// otherwise, <b>false</b>.</returns>
		public bool ShouldSerializeEnableEditInPlace() {
			RootCalendar root = Parent as RootCalendar;
			return root==null || root.EnableEditInPlace!=EnableEditInPlace;
		}

		/// <summary>
		/// Determines the way a calendar is printed (elements to be printed).
		/// </summary>
		/// <remarks>
		/// Setting a way to prinf a calendar affects the operation of 
		/// <see cref="BaseCalendar.Print"/> method.
		/// </remarks>
		[Category("Appearance")]
		public PrintingMode PrintingMode 
		{
			get { return printingMode; }
			set { printingMode = value; }
		}

		/// <summary>
		/// Indicates whether the <see cref="PrintingMode"/> property should be persisted.
		/// </summary>
		/// <remarks>
		/// You typically use this method if you are creating a designer for <see cref="RootCalendar"/>.
		/// </remarks>
		/// <returns><b>true</b> if the property value has changed from its default; 
		/// otherwise, <b>false</b>.</returns>
		public bool ShouldSerializePrintingMode() 
		{
			return !(Parent is RootCalendar) && PrintingMode!=PrintingMode.Standard;
		}

		/// <summary>
		/// Resets the <see cref="PrintingMode"/> property to its default value.
		/// </summary>
		public void ResetPrintingMode() 
		{
			PrintingMode = PrintingMode.Standard;
		}

		private PrintingMode printingMode = PrintingMode.Standard;


		#endregion

		#region Events

		/// <summary>
		/// Called, when new appointments are to be read into <see cref="Appointments"/> collection.
		/// </summary>
		/// <remarks>
		/// Event allows for succesive populating a control with objects
		/// <see cref="Appointment"/> while traversing a calendar. Objects can be
		/// added to existing ones in a collection <see cref="Appointments"/> or
		/// a collection can be fully reloaded.
		/// </remarks>
		[Category("Data")]
		[Description("Called, when new appointments are to be read into Appointments collection.")]
		public event RequestAppointmentsEventHandler RequestAppointments;

		/// <summary>
		///	Raises the <see cref="RequestAppointments"/> event.
		/// </summary>
		/// <param name="args">An <see cref="EventArgs"/> that contains the event data.</param>
		protected virtual void OnRequestAppointments(RequestAppointmentsEventArgs args) {
			if (RequestAppointments!=null)
				RequestAppointments(this, args);
			RootCalendar bc = Parent as RootCalendar;
			if (bc!=null)
				bc.OnRequestAppointments(args);
		}

		/// <summary>
		/// Called upon an appointment datetime or text has changed.
		/// </summary>
		/// <remarks>
		/// Called when a change of date/time or resource occurred as a result of drag&amp;drop operation. 
		/// An event is called also, when a <see cref="Appointment.Text"/> property is changed as 
		/// a result of editing a text. <para/>
		/// When this event occurs, it can be assumed that an appointment's data has changed in 
		/// a way requiring saving changed data to a database.
		/// </remarks>
		[Category("Action")]
		[Description("Called upon an appointment datetime or text has changed.")]
		public event AppointmentEventHandler AppointmentChanged;

		/// <summary>
		///	Raises the <see cref="AppointmentChanged"/> event.
		/// </summary>
		/// <param name="args">An <see cref="EventArgs"/> that contains the event data.</param>
		protected virtual void OnAppointmentChanged(AppointmentEventArgs args) {
			if (AppointmentChanged!=null)
				AppointmentChanged(this, args);
			RootCalendar bc = Parent as RootCalendar;
			if (bc!=null)
				bc.OnAppointmentChanged(args);
		}

		/// <summary>
		/// Called upon an appointment object standard open action. 
		/// </summary>
		/// <remarks>
		/// Standard action is called after a double click on a calendar
		/// or after pressing the ENTER key. 
		/// Most frequently a standard action is to open a window containing 
		/// detailed information on data selected in a calendar (a selected period
		/// or a selected appointment). 
		/// </remarks>
		[Category("Action")]
		[Description("Called by an appointment objects standard Open action.")]
		public event EventHandler Default;

		/// <summary>
		///	Raises the <see cref="Default"/> event.
		/// </summary>
		/// <param name="args">An <see cref="EventArgs"/> that contains the event data.</param>
		protected virtual void OnDefault(EventArgs args) {
			if (Default!=null)
				Default(this, args);
			RootCalendar bc = Parent as RootCalendar;
			if (bc!=null)
				bc.OnDefault(args);
		}

		/// <summary>
		/// An event raised before starting an <see cref="Appointment"/> object edit.
		/// </summary>
		/// <remarks>
		/// Inside this event editing of some objects <see cref="Appointment"/> can be blocked
		/// or you can create your own control to edit them.
		/// Blocking the edit is possible by setting an argument's 
		/// <see cref="EditAppointmentEventArgs.Cancel">EditAppointmentEventArgs.Cancel</see>
		/// property to <b>true</b>. Editing will not start then. Standard value of <b>false</b>
		/// allows for editing all the appointments.<para/>
		/// To implement your own edit control you should create that control and assign it to a 
		/// <see cref="EditAppointmentEventArgs.Control">EditAppointmentEventArgs.Control</see>.
		/// It will be added to a collection <see cref="Control.Controls"/> of properties
		/// automatically. Its location and size will also be set.
		/// </remarks>
		/// <example>
		/// An example shows how to block editing of appointment objects, 
		/// starting before 2003.
		/// <code>
		/// void dailyCalendar_Editing(object sender, EditAppointmentEventArgs args) {
		///		args.Cancel = args.Appointment.DateBegin.Year&lt;2003;
		/// }
		/// </code>
		/// <seealso cref="Edited"/>
		/// <seealso cref="EnableEditInPlace"/>
		/// </example>
		public event EditAppointmentEventHandler Editing;

		/// <summary>
		///	Raises the <see cref="Editing"/> event.
		/// </summary>
		/// <param name="args">An <see cref="EditAppointmentEventArgs"/> that contains the event data.</param>
		protected virtual void OnEditing(EditAppointmentEventArgs args) {
			if (Editing!=null)
				Editing(this, args);
			RootCalendar bc = Parent as RootCalendar;
			if (bc!=null)
				bc.OnEditing(args);
		}

		/// <summary>
		/// An event raised after on-screen edit of an appointment object is finished. 
		/// </summary>
		/// <remarks>
		/// This event is commonly used to validate data entered to an object's edit control.
		/// To accomplish this, a <see cref="AppointmentEventArgs.Appointment"/> 
		/// and the control <see cref="EditAppointmentEventArgs.Control"/>
		/// being edited
		/// are passed to the events <see cref="EditAppointmentEventArgs"/> .
		/// Setting an argument's property <see cref="EditAppointmentEventArgs.ContinueEdit"/> 
		/// to <b>true</b> we can force further edit.<para/>
		/// After this event is raised, the data are moved for the edit control to the
		/// <see cref="Appointment.Text"/> property. It is possible to write your own
		/// method moving the data to an <see cref="Appointment"/> object. 
		/// To accomplish this, you should set an argument's 
		/// <see cref="EditAppointmentEventArgs.Control"/> property to <b>null</b>.
		/// </remarks>
		/// <example>
		/// Sample event handler shows a code, that moves the data from a control
		/// to an <see cref="Appointment"/> object.
		/// <code>
		/// void dailyCalendar_Edited(object sender, EditAppointmentEventArgs args) {
		///		if (!args.Cancel) {
		///			args.Appointment.Text = args.Control.Text;
		///			Controls.Remove(args.Control);
		///			args.Control.Dispose();
		///			args.Control = null;
		///		}
		/// }
		/// </code>
		/// <seealso cref="Editing"/>
		/// <seealso cref="EnableEditInPlace"/>
		/// </example>
		public event EditAppointmentEventHandler Edited;

		/// <summary>
		///	Raises the <see cref="Edited"/> event.
		/// </summary>
		/// <param name="args">An <see cref="EditAppointmentEventArgs"/> that contains the event data.</param>
		protected virtual void OnEdited(EditAppointmentEventArgs args) {
			if (Edited!=null)
				Edited(this, args);
			RootCalendar bc = Parent as RootCalendar;
			if (bc!=null)
				bc.OnEdited(args);
		}

		/// <summary>
		/// Provides detailed information on a day displayed in a calendar.
		/// </summary>
		/// <remarks>
		/// An event called for every day displayed in a calendar. 
		/// It allows to set additional data for advanced presentation
		/// of a day. Setting a holiday, individual colors for a day,
		/// and an hour a working day begins and ends.<para/>
		/// An event allows also for setting these parameter separately 
		/// for each resource.
		/// <seealso cref="GetDayInfoEventArgs"/>
		/// </remarks>
		[Category("Appearance")]
		[Description("Provides detailed information on a day displayed in a calendar.")]
		public event GetDayInfoEventHandler GetDayInfo;

		/// <summary>
		///	Raises the <see cref="GetDayInfo"/> event.
		/// </summary>
		/// <param name="args">An <see cref="GetDayInfoEventArgs"/> that contains the event data.</param>
		protected virtual void OnGetDayInfo(GetDayInfoEventArgs args) {
			if (GetDayInfo!=null)
				GetDayInfo(this, args);
			RootCalendar bc = Parent as RootCalendar;
			if (bc!=null)
				bc.OnGetDayInfo(args);
		}

		/// <summary>
		/// Called during merging two adjacent appointments.
		/// </summary>
		/// <remarks>
		/// An event is called after drag&amp;drop operation resulting in two appointments becoming adjacent. 
		/// Calendar detects such situation and tries to merge the two appointments.<para/>
		/// Two appointments are adjacent, when the end of the first one equals the beginning 
		/// of the second one and both have the same value of a property <see cref="Appointment.ResourceIndex"/>.<para/>
		/// An event allows to decide, whether these two appointments should be merged automatically. 
		/// Permission to merge is controlled by an appointment <see cref="MergeAppointmentsEventArgs.Cancel"/>. 
		/// By default, a calendar merges appointments that are linked.
		/// <seealso cref="Appointment.LinkWithNext"/>
		/// </remarks>
		[Category("Action")]
		[Description("Called during merging two adjacent appointments.")]
		public event MergeAppointmentsEventHandler MergeAppointments;

		/// <summary>
		///	Raises the <see cref="MergeAppointments"/> event.
		/// </summary>
		/// <param name="args">An <see cref="MergeAppointmentsEventArgs"/> that contains the event data.</param>
		protected virtual void OnMergeAppointments(MergeAppointmentsEventArgs args) {
			if (MergeAppointments!=null)
				MergeAppointments(this, args);
			RootCalendar bc = Parent as RootCalendar;
			if (bc!=null)
				bc.OnMergeAppointments(args);
		}

		/// <summary>
		/// This event enables you to create your own method of painting an appointment
		/// in a calendar.
		/// </summary>
		/// <remarks>
		/// To paint an  appointment object you use
		/// <see cref="PaintAppointmentEventArgs.Graphics">args.Graphics</see> containing
		/// an object you are painting with and an
		/// <see cref="AppointmentEventArgs.Appointment">args.Appointment</see> determining
		/// the object to be painted.<para/>
		/// If you are preparing your own algorithm to paint an appointment object in a calendar
		/// and you do not want a standard painting algorithm to be called, you should set
		/// <see cref="PaintAppointmentEventArgs.UserPaint">args.UserPaint</see> to a value of
		/// <b>true</b>.<para/>
		/// During painting, a method <see cref="BaseCalendar.Paint"/> containing
		/// a standard implementation of the object's paint method
		/// can be called.
		/// </remarks>
		/// <example>
		/// An example shows how to write an appointment's Paint delegate
		/// that would block a standard
		/// method of painting a day, and invoke it itself.
		/// <code>
		/// private void cal_PaintAppointment(object sender, ComponentGo.Calendars.PaintAppointmentEventArgs args) {
		///		args.UserPaint = false;
		///		cal.Paint(args.Graphics, args.Appointment);
		///	}
		///	</code>
		/// </example>
		[Category("Appearance")]
		[Description("This event enable you to create your own method of painting an appointment in a calendar.")]
		public event PaintAppointmentEventHandler PaintAppointment;

		/// <summary>
		///	Raises the <see cref="PaintAppointment"/> event.
		/// </summary>
		/// <param name="args">An <see cref="PaintAppointmentEventArgs"/> that contains the event data.</param>
		protected void OnPaintAppointment(PaintAppointmentEventArgs args) {
			if (PaintAppointment!=null)
				PaintAppointment(this, args);
			RootCalendar bc = Parent as RootCalendar;
			if (bc!=null)
				bc.OnPaintAppointment(args);
		}

		/// <summary>
		/// An event raised, when selected days or appointments are changed.
		/// </summary>
		/// <remarks>
		/// An event is raised, when a selected time period in a calendar is changed.
		/// A property <see cref="AppointmentEventArgs.Appointment">args.Appointment</see>
		/// ts ten equal to  <b>null</b>, and an <b>args</b> object is equal to 
		/// <see cref="AppointmentEventArgs.Empty"/>.<para/>
		/// An event is also raised for each seleted or unselected
		/// appointment object. A property 
		/// <see cref="AppointmentEventArgs.Appointment">args.Appointment</see>
		/// is then an object being changed.
		/// </remarks>
		[Category("Property Changed")]
		[Description("An event raised, when selected days or appointments are changed.")]
		public event AppointmentEventHandler SelectionChanged;

		/// <summary>
		/// Raises the <see cref="SelectionChanged"/> event.
		/// </summary>
		/// <param name="args">An <see cref="AppointmentEventArgs"/> that contains the event data.</param>
		protected internal virtual void OnSelectionChanged(AppointmentEventArgs args) {
			if (SelectionChanged!=null)
				SelectionChanged(this, args);
			RootCalendar bc = Parent as RootCalendar;
			if (bc!=null)
				bc.OnSelectionChanged(args);
		}

		/// <summary>
		/// Called during an appointment split operation.
		/// </summary>
		/// <remarks>
		/// An event can disallow a standard split operation is case a given appointment 
		/// cannot be split or split is handled within the event.
		/// In these cases a value of this property should be set to <b>true</b>.<para/>
		/// A calendar automatically turn a split mode off setting a <see cref="BaseCalendar.SplitMode"/> 
		/// property to <b>false</b>. 
		/// If you want a split mode to be active still, a property <see cref="BaseCalendar.SplitMode"/> 
		/// should be set to <b>true</b> again.
		/// <seealso cref="BaseCalendar.SplitMode"/>
		/// </remarks>
		[Category("Action")]
		[Description("Called during an appointment split operation.")]
		public event SplitAppointmentEventHandler SplitAppointment;

		/// <summary>
		///	Raises the <see cref="SplitAppointment"/> event.
		/// </summary>
		/// <param name="args">An <see cref="SplitAppointmentEventArgs"/> that contains the event data.</param>
		protected virtual void OnSplitAppointment(SplitAppointmentEventArgs args) {
			if (SplitAppointment!=null)
				SplitAppointment(this, args);
			RootCalendar bc = Parent as RootCalendar;
			if (bc!=null)
				bc.OnSplitAppointment(args);
		}

		[Category("Action")]
		public event StartAppointmentDragEventHandler StartAppointmentDrag;

		protected virtual void  OnStartAppointmentDrag(StartAppointmentDragEventArgs args)
		{
			if(StartAppointmentDrag!=null)
				StartAppointmentDrag(this, args);
		}


		/// <summary>
		/// An event raised, after a <see cref="TabularCalendar.FirstDate"/>
		/// or <see cref="DailyCalendar.FirstDate"/> property is changed.
		/// </summary>
		[Category("Property Changed")]
		[Description("An event raised, after a FirstDate property is changed")]
		public event EventHandler FirstDateChanged;

		/// <summary>
		///	Raises the <see cref="FirstDateChanged"/> event.
		/// </summary>
		/// <param name="args">An <see cref="EventArgs"/> that contains the event data.</param>
		protected virtual void OnFirstDateChanged(EventArgs args) {
			if (FirstDateChanged!=null)
				FirstDateChanged(this, args);
			RootCalendar bc = Parent as RootCalendar;
			if (bc!=null)
				bc.OnFirstDateChanged(args);
		}

		#endregion
	}
}

