#region Copyright (c) 2003-2004 ComponentGo!
/*
*********************************************************************
*                                                                   *
*       ComponentGo! Component Library                              *
*                                                                   *
*       Copyright (c) 2003-2004 ComponentGo!                        *
*       ALL RIGHTS RESERVED                                         *
*                                                                   *
*********************************************************************
*/
#endregion

using System;
using System.Runtime.Serialization;
using System.Diagnostics;
using System.Collections;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Data;
using System.Windows.Forms;

namespace ComponentGo.Calendars
{

#if !DAILY

	/// <summary>
	/// TabularCalendar is a base class for <see cref="WeeklyCalendar"/> and <see cref="TabularCalendar"/>.
	/// </summary>
	[ToolboxItem(false)]
	public abstract class TabularCalendar : BaseCalendar {

		#region Types

		/// <summary>
		/// Contains the values indentifying various regions of the control.
		/// </summary>
		/// <remarks>
		/// Used by various methods to determine a region (rectangle), the control 
		/// is inquired about
		/// </remarks>
		public enum Regions {
			/// <summary>
			/// A region representing a bar with texts describing each resource.
			/// </summary>
			ResourcesLabel, 
			
			/// <summary>
			/// Day label containing a day or days located in an upper 
			/// portion of the control.
			/// </summary>
			DaysLabel, 
			
			/// <summary>
			/// Region containing appoinments' rectangles 
			/// laid out with daily resolution, located in an upper portion of 
			/// the control.
			/// </summary>
			DaysWorkspace, 
		}

		#endregion

		#region General

		internal TabularCalendar(RootCalendar root, bool isHScroll, bool isVScroll) : base(root, isHScroll, isVScroll) {
			Size = new Size(200, 200);
			this.firstDate = GetRoundedDate(DateTime.Today);
			this.visibleWeeks = DefaultVisibleWeeks;
		}

		#endregion

		#region Properties 

		#region Hide unused

		/// <summary>
		/// Sets the hour resolution, that is the resolution the hours are displayed with.
		/// </summary>
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		[Browsable(false)]
		public new HoursResolutions HoursResolution {
			get { return base.HoursResolution; }
		}

		/// <summary>
		/// The first working hour within a day.
		/// </summary>
		/// <remarks>
		/// Working hours are displayed in a calendar with a  <see cref="BaseCalendar.WorkHourColor"/> color.
		/// <seealso cref="WorkHourEnd"/>
		/// <seealso cref="BaseCalendar.FreeDay"/>
		/// </remarks>
		/// <example>
		/// The example shows how to set working hours from 6:00 to 14:00.
		/// <code>
		/// dailyCalendar.WorkHourBegin = new TimeSpan(6, 0, 0);
		/// dailyCalendar.WorkHourEnd = new TimeSpan(14, 0, 0);
		/// </code>
		/// </example>
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		[Browsable(false)]
		public new TimeSpan WorkHourBegin {
			get { return base.WorkHourBegin; }
		}

		/// <summary>
		/// The last working hour within a day.
		/// </summary>
		/// <remarks>
		/// Working hours are displayed in a calendar with a  <see cref="BaseCalendar.WorkHourColor"/> color.
		/// <seealso cref="WorkHourBegin"/>
		/// <seealso cref="BaseCalendar.FreeDay"/>
		/// </remarks>
		/// <example>
		/// The example shows how to set working hours from 6:00 to 14:00.
		/// <code>
		/// dailyCalendar.WorkHourBegin = new TimeSpan(6, 0, 0);
		/// dailyCalendar.WorkHourEnd = new TimeSpan(14, 0, 0);
		/// </code>
		/// </example>
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		[Browsable(false)]
		public new TimeSpan WorkHourEnd {
			get { return base.WorkHourEnd; }
		}

		#endregion
		
		#region DaysLabelColor

		Color daysLabelColor = SystemColors.Control;
		Color daysLabelColorSecond = SystemColors.Control;

		/// <summary>
		/// Day label color.
		/// </summary>
		/// <remarks>
		/// Day label is located in the upper portion of a control.
		/// </remarks>
		[Category("Colors")]
		[Description("Day label color")]
		public Color DaysLabelColor {
			get { return daysLabelColor; }
			set {
				if (daysLabelColor==value) return;
				if (daysLabelColor==daysLabelColorSecond)
					daysLabelColorSecond = value;
				daysLabelColor = value;
				Invalidate();
			}
		}

		/// <summary>
		/// Indicates whether the <see cref="DaysLabelColor"/> property should be persisted.
		/// </summary>
		/// <returns><b>true</b> if the property value has changed from its 
		/// default; otherwise, <b>false</b>.</returns>
		/// <remarks>You typically use this method if you are creating a designer
		/// for <see cref="TabularCalendar"/>.
		/// </remarks>
		public bool ShouldSerializeDaysLabelColor() {
			return daysLabelColor!=SystemColors.Control;
		}

		/// <summary>
		/// Second color for a <see cref="DaysLabelColor"/> property used for linear gradient
		/// </summary>
		[Category("Colors")]
		[Description("Second color for a day label property used for linear gradient")]
		public Color DaysLabelColorSecond {
			get { return daysLabelColorSecond; }
			set {
				if (daysLabelColorSecond==value) return;
				daysLabelColorSecond = value;
				Invalidate();
			}
		}

		/// <summary>
		/// Indicates whether the <see cref="DaysLabelColorSecond"/> property should be persisted.
		/// </summary>
		/// <returns><b>true</b> if the property value has changed from its 
		/// default; otherwise, <b>false</b>.</returns>
		/// <remarks>You typically use this method if you are creating a designer
		/// for <see cref="TabularCalendar"/>.
		/// </remarks>
		public bool ShouldSerializeDaysLabelColorSecond() {
			return daysLabelColor!=daysLabelColorSecond;
		}

		#endregion

		#region ResourcesLabelColor

		Color resourcesLabelColor = SystemColors.Control;
		Color resourcesLabelColorSecond = SystemColors.Control;

		/// <summary>
		/// Resource label color.
		/// </summary>
		/// <remarks>
		/// Resource label is located in the upper portion of a control.
		/// </remarks>
		[Category("Colors")]
		[Description("Resource label color")]
		public Color ResourcesLabelColor {
			get { return resourcesLabelColor; }
			set {
				if (resourcesLabelColor==value) return;
				if (resourcesLabelColor==resourcesLabelColorSecond)
					resourcesLabelColorSecond = value;
				resourcesLabelColor = value;
				Invalidate();
			}
		}

		/// <summary>
		/// Indicates whether the <see cref="ResourcesLabelColor"/> property should be persisted.
		/// </summary>
		/// <returns><b>true</b> if the property value has changed from its 
		/// default; otherwise, <b>false</b>.</returns>
		/// <remarks>You typically use this method if you are creating a designer
		/// for <see cref="TabularCalendar"/>.
		/// </remarks>
		public bool ShouldSerializeResourcesLabelColor() {
			return resourcesLabelColor!=SystemColors.Control;
		}

		/// <summary>
		/// Second color for a <see cref="ResourcesLabelColor"/> property used for linear gradient
		/// </summary>
		[Category("Colors")]
		[Description("Second color for a resource label property used for linear gradient")]
		public Color ResourcesLabelColorSecond {
			get { return resourcesLabelColorSecond; }
			set {
				if (resourcesLabelColorSecond==value) return;
				resourcesLabelColorSecond = value;
				Invalidate();
			}
		}

		/// <summary>
		/// Indicates whether the <see cref="ResourcesLabelColorSecond"/> property should be persisted.
		/// </summary>
		/// <returns><b>true</b> if the property value has changed from its 
		/// default; otherwise, <b>false</b>.</returns>
		/// <remarks>You typically use this method if you are creating a designer
		/// for <see cref="TabularCalendar"/>.
		/// </remarks>
		public bool ShouldSerializeResourcesLabelColorSecond() {
			return resourcesLabelColor!=resourcesLabelColorSecond;
		}

		#endregion

		#region SelectedAppointmentColor

		/// <summary>
		/// Selected appointments color.
		/// </summary>
		[Category("Colors")]
		[Description("Selected appointments color.")]
		public new Color SelectedAppointmentColor {
			get { return base.SelectedAppointmentColor; }
			set { base.SelectedAppointmentColor = value; }
		}

		/// <summary>
		/// Indicates whether the <see cref="SelectedAppointmentColor"/> property should be persisted.
		/// </summary>
		/// <returns><b>true</b> if the property value has changed from its 
		/// default; otherwise, <b>false</b>.</returns>
		/// <remarks>You typically use this method if you are creating a designer
		/// for <see cref="TabularCalendar"/>.
		/// </remarks>
		public new bool ShouldSerializeSelectedAppointmentColor() {
			return SelectedAppointmentColor!=SystemColors.Highlight;
		}

		#endregion

		#region VisibleWeeks

		/// <summary>
		/// An event raised, after a <see cref="VisibleWeeks"/> property is changed.
		/// </summary>
		[Category("Property Changed")]
		[Description("An event raised, after a VisibleColumns property is changed")]
		public event EventHandler VisibleWeeksChanged;

		/// <summary>
		///	Raises the <see cref="VisibleWeeksChanged"/> event.
		/// </summary>
		/// <param name="args">An <see cref="EventArgs"/> that contains the event data.</param>
		protected virtual void OnVisibleWeeksChanged(EventArgs args) {
			if (VisibleWeeksChanged!=null)
				VisibleWeeksChanged(this, args);
		}

		int visibleWeeks;

		/// <summary>
		/// A number of weeks visible in a calendar.
		/// </summary>
		/// <remarks>
		/// Determines a number of calendar weeks.
		/// </remarks>
		[Category("Appearance")]
		[Description("A number of weeks displayed simultaneously in a calendar.")]
		public int VisibleWeeks {
			get { return visibleWeeks; }
			set {
				if (visibleWeeks==value) return;
				if (value<1) 
					throw new ArgumentException();
				visibleWeeks = value;
				OnVisibleWeeksChanged(EventArgs.Empty);
				Resync(true);
			}
		}

		/// <summary>
		/// Indicates whether the <see cref="VisibleWeeks"/> property should be persisted.
		/// </summary>
		/// <returns><b>true</b> if the property value has changed from its 
		/// default; otherwise, <b>false</b>.</returns>
		/// <remarks>You typically use this method if you are creating a designer
		/// for <see cref="TabularCalendar"/>.
		/// </remarks>
		public bool ShouldSerializeVisibleWeeks() {
			return VisibleWeeks!=DefaultVisibleWeeks;
		}

		internal abstract int DefaultVisibleWeeks { get; }

		#endregion

		#region Align

		ContentAlignment textAlign = ContentAlignment.TopLeft;

		/// <summary>
		/// Gets or sets the alignment of text in the appointment box.
		/// </summary>
		/// <value>
		/// One of the <see cref="ContentAlignment"/> values. The default is 
		/// <see cref="ContentAlignment.TopLeft"/>.
		/// </value>
		[DefaultValue(ContentAlignment.TopLeft)]
		[Category("Appearance")]
		[Description("Gets or sets the alignment of text in the appointment box.")]
		public ContentAlignment TextAlign {
			get {
				return textAlign;
			}
			set {
				if (textAlign==value) return;
				textAlign = value;
				Invalidate();
			}
		}

		ContentAlignment textAllDayAlign = ContentAlignment.TopCenter;

		/// <summary>
		/// Gets or sets the alignment of text in the appointment of the all day type box.
		/// </summary>
		/// <value>
		/// One of the <see cref="ContentAlignment"/> values. The default is 
		/// <see cref="ContentAlignment.TopCenter"/>.
		/// </value>
		[DefaultValue(ContentAlignment.TopCenter)]
		[Category("Appearance")]
		[Description("Gets or sets the alignment of text in the appointment of the all day type box.")]
		public ContentAlignment TextAllDayAlign {
			get {
				return textAllDayAlign;
			}
			set {
				if (textAllDayAlign==value) return;
				textAllDayAlign = value;
				Invalidate();
			}
		}

		ContentAlignment textDayAlign = ContentAlignment.MiddleRight;

		/// <summary>
		/// Gets or sets the alignment of text in days label boxes.
		/// </summary>
		/// <value>
		/// One of the <see cref="ContentAlignment"/> values. The default is 
		/// <see cref="ContentAlignment.MiddleRight"/>.
		/// </value>
		[DefaultValue(ContentAlignment.MiddleRight)]
		[Category("Appearance")]
		[Description("Gets or sets the alignment of text in days label boxes.")]
		public ContentAlignment TextDayAlign {
			get {
				return textDayAlign;
			}
			set {
				if (textDayAlign==value) return;
				textDayAlign = value;
				Invalidate();
			}
		}

		ContentAlignment textResourceAlign = ContentAlignment.MiddleCenter;

		/// <summary>
		/// Gets or sets the alignment of text in resources label boxes.
		/// </summary>
		/// <value>
		/// One of the <see cref="ContentAlignment"/> values. The default is 
		/// <see cref="ContentAlignment.MiddleCenter"/>.
		/// </value>
		[DefaultValue(ContentAlignment.MiddleCenter)]
		[Category("Appearance")]
		[Description("Gets or sets the alignment of text in resources label boxes.")]
		public ContentAlignment TextResourceAlign {
			get {
				return textResourceAlign;
			}
			set {
				if (textResourceAlign==value) return;
				textResourceAlign = value;
				Invalidate();
			}
		}

		#endregion

		#region AllowWeekChange

		bool allowWeekChange = true;

		/// <summary>
		/// Allows for changing a current week while a control is being used.
		/// </summary>
		/// <remarks>
		/// A standard value of <b>true</b> allows the operator to change current week.
		/// Setting a value to <b>false</b> hides a horizontal scrollbar an disallows the operator
		/// to change a date using arrow keys
		/// </remarks>
		[DefaultValue(true)]
		[Category("Behavior")]
		[Description("Allows for changing a current week while a control is being used.")]
		public bool AllowWeekChange {
			get { return allowWeekChange; }
			set {
				if (allowWeekChange==value) return;
				allowWeekChange = value;
				ScrollBars = (ScrollBars & ScrollBars.Vertical)
					| (allowWeekChange ? ScrollBars.Horizontal : ScrollBars.None);
			}
		}

		#endregion

		#region ShowTimeTo

		bool showTimeTo = true;

		/// <summary>
		/// Displays a time an appointment is finished in a calendar.
		/// </summary>
		/// <remarks>
		/// A standard value of <b>true</b> means that after the hour an appointment begins 
		/// on an hour an appoitment is finished is displayed.
		/// Setting a value to <b>false</b> displays only the hour an appoitment begins.
		/// </remarks>
		[DefaultValue(true)]
		[Category("Appearance")]
		[Description("Wywietla w kalendarzu czas zakoczenia appointment.")]
		public bool ShowTimeTo {
			get { return showTimeTo; }
			set {
				if (showTimeTo==value) return;
				showTimeTo = value;
				Invalidate();
			}
		}

		#endregion

		#region ShowAnalogClock

		bool showAnalogClock = false;

		/// <summary>
		/// Information on the hour an appoitment begins and ends displayed as an analog clock.
		/// </summary>
		/// <remarks>
		/// Instead of an hour displayed as a number a small analog clock is painted. 
		/// Am/pm hours are differentiated by reversed face and hands colors.
		/// </remarks>
		[DefaultValue(false)]
		[Category("Appearance")]
		[Description("Information on the hour an appointment begins and ends displayed as an analog clock.")]
		public bool ShowAnalogClock {
			get { return showAnalogClock; }
			set {
				if (showAnalogClock==value) return;
				showAnalogClock = value;
				Invalidate();
			}
		}

		#endregion

		#region FirstDate

		internal abstract DateTime GetRoundedDate(DateTime date);

		DateTime firstDate;

		/// <summary>
		/// The first date visible in a calendar.
		/// </summary>
		/// <remarks>
		/// In case of many dates, this is the first day from the left/top visible in a calendar.
		/// The number of visible days is controlled by a <see cref="VisibleWeeks"/> property.<para/>
		/// A date is always a beginning of a week. 
		/// If it is set to a day in a middle of a week it will be appropriately moved.
		/// </remarks>
		[Category("Data")]
		[Description("The first date visible in a calendar.")]
		public DateTime FirstDate {
			get { return firstDate; }
			set { setFirstDate(value, true); }
		}

		internal void setFirstDate(DateTime value, bool withScrollbars) {
			value = GetRoundedDate(value);
			if (firstDate==value) return;
			if (!stopEdit(false)) return;
			firstDate = value;
			Resync(true);
			if (withScrollbars)
				initScrollBars();
			OnFirstDateChanged(EventArgs.Empty);
		}

		internal DateTime LastDate {
			get {
				return FirstDate.AddDays(7*VisibleWeeks-1);
			}
		}

		/// <summary>
		/// Indicates whether the <see cref="FirstDate"/> property should be persisted.
		/// </summary>
		/// <returns><b>true</b> if the property value has changed from its 
		/// default; otherwise, <b>false</b>.</returns>
		/// <remarks>You typically use this method if you are creating a designer
		/// for <see cref="WeeklyCalendar"/>.
		/// </remarks>
		public bool ShouldSerializeFirstDate() {
			return FirstDate!=GetRoundedDate(DateTime.Today);
		}

		internal override DateTime VisibleFirstDateTime {
			get { return FirstDate; }
			set { FirstDate = value; }
		}

		internal override TimeSpan VisibleTimeSpan {
			get { return TimeSpan.FromDays(VisibleWeeks*7); }
			set { VisibleWeeks = (int)value.TotalDays/7; }
		}

		#endregion

		#endregion

		#region Scrollbars

		internal abstract ScrollBar Scroll { get; }

		internal override void initScrollBars() {
			////////////////////////////////////////////////////////////////
			// Horizontal ScrollBar
			// 
			int win = 6*7;

			int i = (int)(FirstDate-DateZero).TotalDays;
			ScrollBar scroll = Scroll;

			scroll.Minimum = i-win;
			scroll.Maximum = i+win+VisibleWeeks*7;
			scroll.Value = i;
			scroll.LargeChange = VisibleWeeks*7;
			scroll.SmallChange = 7;
		}

		void scroll_Scroll(ScrollEventArgs args) {
			if (args.Type!=ScrollEventType.EndScroll)
				setFirstDate(DateZero.AddDays(args.NewValue), false);
			else {
				ScrollBar scroll = Scroll;
				int v = scroll.Value-scroll.Minimum;
				int w = scroll.Maximum-scroll.Minimum;
				bool change = false;
				if (v<w/10) change = true;
				if (v+scroll.LargeChange>w-w/10) change = true;
				if (change)
					initScrollBars();
			}
		}

		internal override void vScroll_Scroll(object sender, ScrollEventArgs args) {
			scroll_Scroll(args);
		}

		internal override void hScroll_Scroll(object sender, ScrollEventArgs args) {
			scroll_Scroll(args);
		}

		/// <summary>
		/// Forces visibility of a given period in a calendar.
		/// </summary>
		/// <param name="dateFrom">A beginning of a period, that is to be visible in a calendar.</param>
		/// <param name="dateTo">An end of a period, that is to be visible in a calendar.</param>
		/// <param name="resourceIdx">A index of resource, that is to be visible in a calendar.</param>
		/// <remarks>
		/// It is possible that a given period wil not be wholly visible, because a calendar
		/// is too small. In this case a largest possible portion of a period will be visible.
		/// </remarks>
		public override void EnsureVisible(DateTime dateFrom, DateTime dateTo, int resourceIdx) {
			dateFrom = dateFrom.Date;
			dateTo = dateTo.Date;

			if (dateFrom<FirstDate)
				setFirstDate(dateFrom, true);
			else if (dateTo>LastDate.AddDays(1))
				setFirstDate(dateTo.AddMilliseconds(-1).AddDays(-(VisibleWeeks-1)*7), true);
		}

		#endregion

		#region Appointments

		internal HybridDictionary [][] items;
		internal ArrayList sortedItems;
		static readonly HybridDictionary EmptyDictionary = new HybridDictionary();

		internal HybridDictionary GetItems(int idx, int resourceIndex) {
			if (items==null)
				initItems();
			HybridDictionary result = null;
			HybridDictionary [] arr = items[Math.Min(resourceIndex, ResCount-1)];
			if (arr!=null && 0<=idx && idx<arr.Length)
				result = arr[idx];
			if (result==null)
				result = EmptyDictionary;
			return result;
		}

		void initItems() {
			forceAppointments();
			
			items = new HybridDictionary[ResCount][];
			sortedItems = new ArrayList();

			foreach (Appointment ap in Appointments)
				if (ap.AllDay)
					addAp(ap);
			foreach (Appointment ap in Appointments)
				if (!ap.AllDay && ap.DateBegin.Date!=ap.DateEnd.Date)
					addAp(ap);
			foreach (Appointment ap in Appointments)
				if (!ap.AllDay && ap.DateBegin.Date==ap.DateEnd.Date)
					addAp(ap);
		}

		internal abstract void addAp(Appointment ap);

		public override void Resync(bool recalc) {
			if (recalc) {
				items = null;
				sortedItems = null;
			}
			Invalidate();
		}

		#endregion

		#region Keyboard

		internal override Appointment getNext(Appointment ap) {
			int i = sortedItems.IndexOf(ap)+1;
			if (i<sortedItems.Count)
				return (Appointment)sortedItems[i];
			return null;
		}

		internal override Appointment getPrevious(Appointment ap) {
			if (ap==null) {
				if (sortedItems.Count>0)
					return (Appointment)sortedItems[sortedItems.Count-1];
			}
			else {
				int i = sortedItems.IndexOf(ap);
				if (i>0) 
					return (Appointment)sortedItems[i-1];
			}
			return null;
		}

		internal override void checkSelected() {
			if (SelectedDateBegin==DateTime.MinValue) {
				SelectedDateBegin = FirstDate;
				SelectedDateEnd = FirstDate;
			}
		}

		internal override void updateSelection(DateTime dateBegin, DateTime dateEnd, int resource, bool growEnd) {
			if (!AllowWeekChange) {
				if (dateBegin.Date<FirstDate) return;
				if (dateEnd>LastDate.AddDays(1)) return;
			}
			base.updateSelection(dateBegin, dateEnd, resource, growEnd);
		}
		
		#endregion

		#region Mouse

		internal override void mouseClick(System.Windows.Forms.MouseEventArgs e, bool grow) {
			Point pt = new Point(e.X, e.Y);

			DateTime date = HitDateTimeTest(pt);
			if (date==DateTime.MinValue)
				date = SelectedDateBegin;

			int resourceIdx = HitResourceTest(pt);
			if (resourceIdx<0) 
				resourceIdx = SelectedResource;
			
			Appointments.UnselectAll();
			moveCursor(grow, date, OneDay, resourceIdx);
		}

		internal override Cursor getCursorAppearance(Point pt, Appointment ap) {
			return Cursors.SizeWE;
		}

		#endregion

		#region Hit

		internal override AppRegion HitAppointmentRegion(Appointment ap, Point pt) {
			Rectangle [] rts = GetAppointmentRectangles(ap);

			bool isFirst = getDateIdx(ap.dateStart, false)>=0;
			bool isLast = getDateIdx(ap.dateStop, true)<7*VisibleWeeks;

			for (int i = 0; i<rts.Length; ++i) {
				Rectangle rt = rts[i];
				if (rt.Contains(pt)) {
					if (ap.AllDay) {
						if (isFirst && i==0 && rt.Left<=pt.X && pt.X<rt.Left+3)
							return AppRegion.BeginMargin;
						if (isLast && i==rts.Length-1 && rt.Right-3<=pt.X && pt.X<rt.Right)
							return AppRegion.EndMargin;
					}
					return AppRegion.Content;

				}
			}
			return AppRegion.None;
		}

		#endregion

		#region PaintAppointment

		internal void paintAppointment(Graphics g, Appointment ap) {
			if (getDateIdx(ap.dateStop, true)<0) return;
			if (getDateIdx(ap.dateStart, false)>=7*VisibleWeeks) return;
			InvokePaintAppointment(new PaintAppointmentEventArgs(g, ap));
		}

		const int requiredWidth = 40;

		/// <summary>
		/// Draws an appointment in a selected graphics object.
		/// </summary>
		/// <param name="graphics">Graphics object used to display an appointment.</param>
		/// <param name="ap">An appointment drawn in a calendar. </param>
		public override void Paint(Graphics graphics, Appointment ap) 
		{
			Color textColor = ap.Selected ? SelectedAppointmentTextColor : 
				(ap.ForeColor!=Color.Empty ? ap.ForeColor : ForeColor);
			Color color = ap.Selected ? SelectedAppointmentColor : 
				(ap.BackColor!=Color.Empty ? ap.BackColor : BackColor);
			Color progressColor = ap.ProgressColor;
			if (ap.IsGhost) 
			{
				textColor = Color.FromArgb(102, textColor);
				color = Color.FromArgb(102, color);
				progressColor = Color.FromArgb(102, progressColor);
			}

			using (StringFormat textFormat = new StringFormat(StringFormatFlags.DisplayFormatControl))
			using (StringFormat textFormatHM = new StringFormat(StringFormatFlags.DisplayFormatControl))
			using (Brush textBrush = new SolidBrush(textColor))
			using (Pen linePen = new Pen(ap.AllDay ? ControlPaint.Dark(DaysLabelColorSecond) : ap.GetLineColor()))
			using (SolidBrush brush = new SolidBrush(color)) 
			{
				bool isFirst = getDateIdx(ap.dateStart, false)>=0;
				bool isLast = getDateIdx(ap.dateStop, true)<7*VisibleWeeks;
				bool multiDay = ap.DateBegin.Date!=ap.DateEnd.Date;

				CalendarTools.SetAlignment(textFormat, 
					ap.AllDay || multiDay ? TextAllDayAlign : TextAlign);
				textFormat.Trimming = StringTrimming.EllipsisCharacter;
				textFormat.FormatFlags |= StringFormatFlags.NoWrap;
				switch (TextAlign) {
					case ContentAlignment.BottomCenter:
					case ContentAlignment.BottomLeft:
					case ContentAlignment.BottomRight:
						textFormatHM.LineAlignment = StringAlignment.Far;
						break;
					case ContentAlignment.MiddleCenter:
					case ContentAlignment.MiddleLeft:
					case ContentAlignment.MiddleRight:
						textFormatHM.LineAlignment = StringAlignment.Center;
						break;
					case ContentAlignment.TopCenter:
					case ContentAlignment.TopLeft:
					case ContentAlignment.TopRight:
						textFormatHM.LineAlignment = StringAlignment.Near;
						break;
				}

				Rectangle [] rects = GetAppointmentRectangles(ap);
				for (int i = 0; i<rects.Length; ++i) {
					Rectangle rt = rects[i];
					if (rt==Rectangle.Empty) continue;

					/////////////////////////////////////////////////////
					// Background of a frame
					// 
					int deltaX = 0;
					if (ap.Progress>0) {
						deltaX = rt.Width*ap.Progress/100;
						using (Brush progressBrush = new SolidBrush(progressColor))
							graphics.FillRectangle(progressBrush, rt.Left, rt.Top, deltaX, rt.Height);
					}
					if (ap.AllDay || multiDay) {
						graphics.FillRectangle(brush, rt.Left+deltaX, rt.Top, rt.Width-deltaX, rt.Height);
						graphics.DrawLine(linePen, rt.Left, rt.Top, rt.Right, rt.Top);
						graphics.DrawLine(linePen, rt.Left, rt.Bottom, rt.Right, rt.Bottom);
						if (isFirst && i==0)
							graphics.DrawLine(linePen, rt.Left, rt.Top, rt.Left, rt.Bottom);
						if (isLast && i==rects.Length-1)
							graphics.DrawLine(linePen, rt.Right, rt.Top, rt.Right, rt.Bottom);
					}
					else if (ap.Selected)
						graphics.FillRectangle(brush, rt.Left+deltaX, rt.Top, rt.Width-deltaX, rt.Height);

					//////////////////////////////////////////////////////
					// Text
					// 	
					int x2 = rt.X+1, xx2 = rt.Right, y2 = rt.Y+1;
					rt.Inflate(-2, -2);

					Font f = ap.Font==null ? Font : ap.Font;

					if (!ap.AllDay) {
						if (multiDay) {
							if (isFirst && i==0) {
								if (rt.Width-timeMargin>requiredWidth) {
									if (ShowAnalogClock)
										CalendarTools.PaintClock(graphics, ap.DateBegin, x2, y2, appointmentHeight-4); 
									else
										graphics.DrawString(ap.DateBegin.ToShortTimeString(), f, textBrush, rt, textFormatHM);
									rt.X += timeMargin;
									rt.Width -= timeMargin;
								}
							}
							if (isLast && i==rects.Length-1) {
								if (rt.Width-timeMargin>requiredWidth) {
									if (ShowAnalogClock)
										CalendarTools.PaintClock(graphics, ap.DateEnd, xx2-appointmentHeight+3, y2, appointmentHeight-4); 
									else {
										Rectangle rt2 = rt;
										rt2.X = rt.Right-timeMargin;
										rt2.Width = timeMargin+4;
										graphics.DrawString(ap.DateEnd.ToShortTimeString(), f, textBrush, rt2, textFormatHM);
									}
									rt.Width -= timeMargin;
								}
							}
						}
						else {
							if (rt.Width-timeMargin>requiredWidth) {
								bool showTo = ShowTimeTo;
								int m = largeTimeMargin;
								if (rt.Width-largeTimeMargin<=requiredWidth) {
									m = timeMargin;
									showTo = false;
								}
								if (ShowAnalogClock) {
									CalendarTools.PaintClock(graphics, ap.DateBegin, x2, y2, appointmentHeight-4); 
									if (showTo)
										CalendarTools.PaintClock(graphics, ap.DateEnd, x2+timeMargin+1, y2, appointmentHeight-4); 
								}
								else {
									string s = showTo ? "{0:t} {1:t}" : "{0:t}";
									s = string.Format(s, ap.DateBegin, ap.DateEnd);

									graphics.FillRectangle(brush, rt);
									graphics.DrawRectangle(linePen, rt);
									graphics.DrawString(s, f, textBrush, rt, textFormatHM);
								}

								rt.X += m;
								rt.Width -= m;
							}
						}
					}
					
					if (rt.Width>0)			 
						graphics.DrawString(" "+ap.Text, f, textBrush, rt, textFormat);
				}
			}
		}

		#endregion

		#region Implementation

		internal bool isResourceBar {
			get { return Resources.Count>0; }
		}

		internal int getDateIdx(DateTime date, bool prev) {
			if (date==DateTime.MinValue) return -1;
			if (prev)
				date = date.AddMilliseconds(-1);
			return (int)(date.Date-FirstDate).TotalDays;
		}

		/// <summary>
		/// Calculates a rectangle, containing text located inside an appointment's rectangle.
		/// </summary>
		/// <remarks>
		///	Rectangle of appointment's text is also used as a rectangle of a text editor used to edit the text.
		/// </remarks>
		/// <param name="ap">Appointment a rectangle is calculated for.</param>
		/// <param name="eip">Determines whether a rectangle used by EiP <see cref="BaseCalendar.EditControl"/> should be calculated.</param>
		/// <returns>A rectangle calculated.</returns>
		public override Rectangle GetAppointmentTextRectangle(Appointment ap, bool eip) {
			Rectangle rt = GetAppointmentRectangles(ap)[0];
			rt.Inflate(-1, -1);
			if (ap.AllDay) {
				rt.X += 1;
				rt.Y += 1;
				rt.Width -= 1;
				rt.Height -= 1;
			}
			else {
				rt.X += 3;
				rt.Width -= 3;
			}
			if (!ap.AllDay) {
				int m = ap.DateBegin.Date==ap.DateEnd.Date ? largeTimeMargin : timeMargin;
				if (rt.Width-m<=requiredWidth) {
					m = timeMargin;
					if (rt.Width-m<=requiredWidth)
						m = 0;
				}
				rt.X += m;
				rt.Width -= m;
			}
			return rt;
		}

		internal override RequestAppointmentsEventArgs CreateRequestAppointmentsArgs() {
			return new RequestAppointmentsEventArgs(
				FirstDate, LastDate,
				0, int.MaxValue);
		}

		#region Margin

		internal int timeMargin = 20;
		internal const int appointmentHeight = 20;

		internal void initMargin(Graphics g) {
			if (ShowAnalogClock)
				timeMargin = appointmentHeight-2;
			else
				timeMargin = (int)Math.Floor(g.MeasureString(
					new DateTime(2000, 1, 1, 23, 59, 59).ToShortTimeString(),
					Font).Width)-3;
		}

		internal int largeTimeMargin {
			get { return ShowTimeTo ? 2*timeMargin : timeMargin; }
		}

		#endregion

		#endregion
	}

#endif

}
