#region Copyright (c) 2003-2004 ComponentGo!
/*
*********************************************************************
*                                                                   *
*       ComponentGo! Component Library                              *
*                                                                   *
*       Copyright (c) 2003-2004 ComponentGo!                        *
*       ALL RIGHTS RESERVED                                         *
*                                                                   *
*********************************************************************
*/
#endregion

using System;
using System.Runtime.Serialization;
using System.Diagnostics;
using System.Collections;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Data;
using System.Windows.Forms;

namespace ComponentGo.Calendars {

#if !DAILY

	/// <summary>
	/// Weekly calendar control. Can be placed on a form.
	/// </summary>
	/// <remarks>
	/// A calendar displays days as a week, 
	/// where the first column is Monday-Wednesday and the second Thursday-Sunday.
	/// It is possible to display multiple weeks next to each other.
	/// If a calendar contains many resurces, days are additionally divided into subsequent columns, 
	/// one for each resource.<para/>
	/// Saturday and Sunday are placed in the last field.
	/// </remarks>
	[Designer(typeof(Design.BaseCalendarDesigner), typeof(System.ComponentModel.Design.IDesigner))]
	[ToolboxItem(true)]
	[System.Runtime.InteropServices.ComVisible(true)]
	[System.Runtime.InteropServices.Guid("A69A2FEB-5C06-4d93-A739-66ED40AE4A03")]
	public class WeeklyCalendar : TabularCalendar {
		
		#region General

		/// <summary>
		/// Constructor creating a calendar control. 
		/// Properties are initialized to standard values.
		/// </summary>
		public WeeklyCalendar() : this(null) {
		}

		internal WeeklyCalendar(RootCalendar root) : base(root, true, false) {
			SelectedAppointmentColor = SystemColors.Highlight;
		}

		internal override int DefaultVisibleWeeks {
			get { return 1; }
		}

		internal override DateTime GetRoundedDate(DateTime date) {
			return CalendarTools.GetMondayDate(date);
		}

		internal override ScrollBar Scroll {
			get { return hScroll; }
		}

		internal override void addAp(Appointment ap) {
			if (ap.ResourceIndex>=items.Length) return;

			int idxFrom = getDateIdx(ap.dateStart, false);
			int idxTo = getDateIdx(ap.dateStop, true);
			if (idxTo<0 || 7*VisibleWeeks<=idxFrom) return;

			sortedItems.Add(ap);

			HybridDictionary [] arr = items[ap.ResourceIndex];
			if (arr==null)
				items[ap.ResourceIndex] = arr = new HybridDictionary[7*VisibleWeeks];
			for (int i = Math.Max(0, idxFrom); i<=Math.Min(7*VisibleWeeks-1, idxTo); ++i) {
				HybridDictionary t = arr[i];
				if (t==null)
					arr[i] = t = new HybridDictionary();
				t[ap] = t.Count;
			}
		}

		#endregion

		#region DateLabelFormat

		string dateLabelFormat = "{0:m}";

		/// <summary>
		/// Day display format in the upper portion of a calendar.
		/// </summary>
		/// <remarks>
		/// Standard property value {0:m} means that a day label consists of a day number 
		/// and a full month name. The property value is passed to a 
		/// <see cref="String.Format">String.Format</see> method,
		/// formatting a displayed text. The second parameter is a  <see cref="DateTime"/> 
		/// class object denoting a displayed day.
		/// </remarks>
		/// <example>
		/// An example shows setting a property to display a full date with a string
		/// <b>day</b> at the end.
		/// <code>
		/// dailyCalendar.DateLabelFormat = "{0} day";
		/// </code>
		/// </example>
		[DefaultValue("{0:m}")]
		[Category("Appearance")]
		[Description("Day display format in the upper portion of a calendar")]
		public string DateLabelFormat {
			get { return dateLabelFormat; }
			set {
				if (dateLabelFormat==value) return;
				dateLabelFormat = value;
				Invalidate();
			}
		}

		#endregion

		#region Rectangles

		const int labelHeight = 20;

		int clientWidth {
			get { return Width; }
		}

		int NCols { 
			get { return 2*VisibleWeeks*ResCount; }
		}

		/// <summary>
		/// Returns a rectangle containing information on columns for consecutive resources.
		/// </summary>
		/// <remarks>
		/// If a calendar does not contain recources (<see cref="Resource"/>.Count==0) 
		/// a header is not painted and a function returns <see cref="Rectangle.Empty"/>.
		/// </remarks>
		/// <returns>A rectangle object a resource header is painted in.</returns>
		public Rectangle GetResourcesLabelRectangle() {
			if (isResourceBar)
				return new Rectangle(0, 0, clientWidth, labelHeight);
			return Rectangle.Empty;
		}

		/// <summary>
		/// Returns a rectangle containing a main part of a calendar.
		/// </summary>
		/// <remarks>
		/// In this area only day headers and appointments are painted.
		/// </remarks>
		/// <returns>A rectangle object cointaining most of a control.</returns>
		public Rectangle GetWorkspaceRectangle() {
			int y = isResourceBar ? labelHeight : 0;
			return new Rectangle(0, y, clientWidth, Height-y-HorizontalScrollBarHeight);
		}

		int GetDaysLabelY(DayOfWeek dow) {
			int y = isResourceBar ? labelHeight : 0;
			y = labelHeight;
			int h = Height-y-HorizontalScrollBarHeight;

			switch (dow) {
				case DayOfWeek.Monday:
				case DayOfWeek.Thursday:
					return y;

				case DayOfWeek.Tuesday:
				case DayOfWeek.Friday:
					return y + h/3;

				case DayOfWeek.Wednesday:
				case DayOfWeek.Saturday:
					return y + 2*h/3;
			}
			return y + 5*h/6;
		}

		int GetColumnX(int column) {
			int ncol = NCols;
			if (column>=ncol)
				return Width;
			return column*Width/ncol;
		}

		static bool IsFirstColumn(DayOfWeek dow) {
			switch (dow) {
				case DayOfWeek.Thursday:
				case DayOfWeek.Friday:
				case DayOfWeek.Saturday:
				case DayOfWeek.Sunday:
					return true;
			}
			return false;
		}

		Rectangle GetBoxRectangle(DayOfWeek dow, int week, int resourceFrom, int resourceTo) {
			int y = GetDaysLabelY(dow);
			int y2;
			if (dow==DayOfWeek.Wednesday || dow==DayOfWeek.Sunday)
				y2 = Height-HorizontalScrollBarHeight;
			else
				y2 = GetDaysLabelY(dow+1);

			int col = 2*week;
			if (IsFirstColumn(dow))
				++col;
			col *= ResCount;

			int x = GetColumnX(col+resourceFrom);
			int x2 = GetColumnX(col+resourceTo);

			return new Rectangle(x, y, x2-x, y2-y);
		}

		/// <summary>
		/// Returns a rectangle a particular day from the calendar is painted in.
		/// </summary>
		/// <remarks>
		/// A day header and appointments in it are painted in this rectangle.
		/// </remarks>
		/// <param name="date">The date a rectangle is calculated for.</param>
		/// <returns>A calculated rectangle.</returns>
		public Rectangle GetDayRectangle(DateTime date) {
			if (FirstDate<=date && date<=LastDate) {
				int week = getDateIdx(date, false)/7;
				return GetBoxRectangle(date.DayOfWeek, week, 0, ResCount);
			}

			return Rectangle.Empty;
		}

		/// <summary>
		/// Returns a rectangle a particular region of a day from the calendar is painted in.
		/// </summary>
		/// <param name="date">The date a rectangle is calculated for.</param>
		/// <param name="region">Region dnia, dla ktrego wyliczony jest rectangle.</param>
		/// <returns>A region of a day a rectangle is calculated for.</returns>
		public Rectangle GetDayRectangle(DateTime date, Regions region) {
			if (region!=Regions.DaysLabel && region!=Regions.DaysWorkspace) 
				throw new ArgumentException("Region is not supported.", "region");

			Rectangle rt = GetDayRectangle(date);
			if (region==Regions.DaysLabel)
				rt.Height = labelHeight;
			else {
				rt.Y += labelHeight;
				rt.Height -= labelHeight;
			}

			return rt;
		}

		/// <summary>
		/// Returns a rectangle, appointment for chosen day and resource are painted in.
		/// </summary>
		/// <param name="date">The date a rectangle is calculated for.</param>
		/// <param name="resourceIndex">, a rectangle is calculated for.</param>
		/// <returns>A calculated rectangle.</returns>
		public Rectangle GetResourceRectangle(DateTime date, int resourceIndex) {
			if (FirstDate<=date && date<=LastDate) {
				int week = ((int)(date-FirstDate).TotalDays)/7;
				Rectangle rt = GetBoxRectangle(date.DayOfWeek, week, resourceIndex, resourceIndex+1);
				rt.Y += labelHeight;
				rt.Height -= labelHeight;
				return rt;
			}

			return Rectangle.Empty;
		}

		/// <summary>
		/// Returns a rectangles on a screen containing a given appointment.
		/// </summary>
		/// <param name="ap">Appointment, the rectangles are returned for.</param>
		/// <returns>A table of rectangles, calculated relative to 
		/// an upper left corner of the calendar control. 
		///	</returns>
		public override Rectangle[] GetAppointmentRectangles(Appointment ap) {
			int idxFrom = Math.Max(getDateIdx(ap.dateStart, false), 0);
			int idxTo = Math.Min(getDateIdx(ap.dateStop, true), 7*VisibleWeeks);
			if (idxFrom>idxTo) return new Rectangle[0];

			Rectangle[] rects = new Rectangle[idxTo-idxFrom+1];

			for (int i = idxFrom; i<=idxTo; ++i) {
				Rectangle rt = Rectangle.Empty;
				DateTime date = FirstDate.AddDays(i);
				HybridDictionary dic = GetItems(i, ap.ResourceIndex);
				object idx = dic[ap];
				if (idx!=null) {
					rt = GetResourceRectangle(date, ap.ResourceIndex);
					int maxY = rt.Bottom;
					rt.Y += appointmentHeight*(int)idx;
					rt.Height = appointmentHeight-2;
					rt.X += 2;
					rt.Width -= 6;
					if (rt.Bottom>maxY)
						rt = Rectangle.Empty;
				}
				rects[i-idxFrom] = rt;
			}

			return rects;
		}

		#endregion

		#region Hits

		/// <summary>
		/// Calculates a calendar region located at given coordinates.
		/// </summary>
		/// <param name="pt">Coordinates relative to a calendar control
		/// the region is calculated for.</param>
		/// <returns>Calculated region of a calendar control.</returns>
		public Regions HitTest(Point pt) {
			if (GetResourcesLabelRectangle().Contains(pt)) return Regions.ResourcesLabel;

			for (DateTime dt = LastDate; dt>=FirstDate; dt = dt.AddDays(-1))
				if (GetDayRectangle(dt, Regions.DaysLabel).Contains(pt))
					return Regions.DaysLabel;

			return Regions.DaysWorkspace;
		}

		/// <summary>
		/// Calculates a day and an hour located at given coordinates.
		/// </summary>
		/// <param name="pt">
		/// Coordinates relative to a calendar control, an hour is calculated for.
		/// </param>
		/// <returns>Structure containing calculated information on day and hour.</returns>
		public override DateTime HitDateTimeTest(Point pt) {
			for (DateTime dt = LastDate; dt>=FirstDate; dt = dt.AddDays(-1))
				if (GetDayRectangle(dt).Contains(pt))
					return dt;

			return DateTime.MinValue;
		}

		/// <summary>
		/// Calculates an index of a resource located at a given point.
		/// </summary>
		/// <remarks>
		/// Method always returns 0.
		/// </remarks>
		/// <param name="pt">Coordinates relative to a calendar control, 
		///		an resource is calculated for.</param>
		/// <returns>Value 0.</returns>
		public override int HitResourceTest(Point pt) {
			for (int i = NCols; --i>=0;) {
				int x = GetColumnX(i);
				if (x<=pt.X)
					return i % ResCount;
			}
			return 0;
		}

		#endregion

		#region Paint

		#region Background

		void paintBackground(Graphics g) {
			Rectangle rect = GetWorkspaceRectangle();
			if (rect.Width<=0 || rect.Height<=0) return;

			CalendarTools.FillRectangle(g, rect, WorkHourColor, WorkHourColorSecond, true);
		}

		#endregion

		#region Resources Label

		void paintResourcesLabel(Graphics g) {
			if (!isResourceBar) return;

			Rectangle rt = GetResourcesLabelRectangle();
			if (rt.Width<=0) return;

			CalendarTools.FillRectangle(g, rt, ResourcesLabelColor, ResourcesLabelColorSecond, true);

			for (int week = VisibleWeeks; --week>=0;) {
				DateTime date = FirstDate.AddDays(7*week);
				for (int resourceIndex = Resources.Count; --resourceIndex>=0;) {
					paintResourceLabel(g, rt, date, date.AddDays(3), resourceIndex);
					paintResourceLabel(g, rt, date.AddDays(3), date.AddDays(7), resourceIndex);
				}
			}
		}

		void paintResourceLabel(Graphics g, Rectangle rt, DateTime d1, DateTime d2, int resourceIndex) {
			Rectangle resRt = GetResourceRectangle(d1, resourceIndex);
			resRt.Y = rt.Y; resRt.Height = rt.Height;
			bool isSelected = SelectedResource==resourceIndex
				&& intersected(d1, d2, SelectedDateBegin, SelectedDateEnd);

			if (isSelected)
				using (Brush selectedBrush = new SolidBrush(SelectedColor))
					g.FillRectangle(selectedBrush, resRt);
	
			ControlPaint.DrawBorder3D(g, resRt);
			resRt.Inflate(-2, -2);
			if (resRt.Width>0) {
				using (StringFormat textFormat = new StringFormat(StringFormatFlags.DisplayFormatControl | StringFormatFlags.NoWrap))
				using (Brush textBrush = new SolidBrush(isSelected ? SelectedAppointmentTextColor : ForeColor)) {
					CalendarTools.SetAlignment(textFormat, TextResourceAlign);
					textFormat.Trimming = StringTrimming.EllipsisWord;
					
					g.DrawString(Resources[resourceIndex].Text, 
						Font, textBrush, resRt, textFormat);
				}
			}
		}

		#endregion

		#region Days Label

		void paintDaysLabel(Graphics g) {
			DateTime today = DateTime.Today;

			using (Pen linePen1 = new Pen(LineColor, 1))
			using (Pen linePen2 = new Pen(ControlPaint.Light(LineColor), 1))
			using (StringFormat textFormat = new StringFormat(StringFormatFlags.DisplayFormatControl | StringFormatFlags.NoWrap))
			using (Brush textSelectedBrush = new SolidBrush(SelectedAppointmentTextColor))
			using (Brush textBrush = new SolidBrush(ForeColor)) {
				CalendarTools.SetAlignment(textFormat, TextDayAlign);
				textFormat.Trimming = StringTrimming.EllipsisWord;

				for (int i = NCols; --i>0;) {
					int x = GetColumnX(i)-1;

					g.DrawLine(
						i % ResCount==0 ? linePen1 : linePen2, 
						x, 0, x, Height);
				}

				for (DateTime date = LastDate; date>=FirstDate; date = date.AddDays(-1)) {
					////////////////////////////////////////////////////////////////
					// Day label
					//
					Rectangle dayRt = GetDayRectangle(date, Regions.DaysLabel);

					bool isSelected = SelectedDateBegin<=date && date<SelectedDateEnd;

					Color secondColor = isSelected ? SelectedColor : DaysLabelColorSecond;
					bool isToday = TodayMarkerVisible && date==today;
					if (isToday)
						secondColor = ControlPaint.LightLight(TodayMarkerColor);
					CalendarTools.FillRectangle(g, dayRt, 
						isSelected ? SelectedColor : DaysLabelColor, secondColor, true);
					if (isToday)
						using (Pen todayPen = new Pen(TodayMarkerColor))
							g.DrawLine(todayPen, dayRt.Left, dayRt.Bottom-3, dayRt.Right, dayRt.Bottom-3);

					ControlPaint.DrawBorder3D(g, dayRt);

					dayRt.Inflate(-2, -2);
					if (dayRt.Width>0)
						g.DrawString(string.Format(DateLabelFormat, date), 
							Font, 
							isSelected ? textSelectedBrush : textBrush, 
							dayRt, textFormat);
				}
			}
		}

		#endregion

		#region Days Workspace

		void paintDaysWorkspace(Graphics g) {
			for (DateTime date = LastDate; date>=FirstDate; date = date.AddDays(-1)) {
				GetDayInfoEventArgs gdi = new GetDayInfoEventArgs(this, date);
				OnGetDayInfo(gdi);

				if (gdi.IndividualWorkColor) 
				{
					Rectangle dayRt = GetDayRectangle(date, Regions.DaysWorkspace);
					CalendarTools.FillRectangle(g, dayRt, gdi.WorkHourColor, gdi.WorkHourColorSecond, true);
				}

				for (int resourceIdx = ResCount; --resourceIdx>=0;) {
					Resource res = resourceIdx>=Resources.Count ? null : Resources[resourceIdx];
					GetDayInfoEventArgs.ResourceInfo ri = 
						gdi.resourcesInfo==null ? null : gdi.resourcesInfo[resourceIdx];

					Color resColor = Color.Empty;
					if (res!=null && res.BackColor!=Color.Empty)
						resColor = res.BackColor;
					else if (gdi.FreeDay)
						resColor = ri==null ? gdi.FreeHourColor : ri.FreeHourColor;

					if (resColor!=Color.Empty) {
						Rectangle rt = GetResourceRectangle(date, resourceIdx);
						using (Brush brush = new SolidBrush(resColor))
							g.FillRectangle(brush, rt);
					}
				}
			}
		}

		#endregion

		/// <summary>
		/// Overloaded. Implementation of a calendar.
		/// </summary>
		/// <param name="pe">Like overloaded</param>
		protected override void OnPaint(PaintEventArgs pe) 
		{
			Graphics g = pe.Graphics;
			initMargin(g);

			paintBackground(g);
			paintDaysWorkspace(g);
			paintDaysLabel(g);
			paintResourcesLabel(g);

			GetItems(0, 0);
			foreach (Appointment ap in Appointments)
					paintAppointment(g, ap);

			/*int d0=0, d1=0, d2=0, d3=0, d4=0, d5=0, d6=0;
			foreach (Appointment ap in Appointments)
			{
				if (ap.dateStart.Date.CompareTo(FirstDate)>=0 && 
					ap.dateStop.Date.CompareTo(FirstDate.AddDays(7))<=0)   
				{
					switch (ap.dateStart.DayOfWeek)
					{
						case DayOfWeek.Sunday: d0++; 
							if (d0<=7)
								paintAppointment(g, ap);
							else
							{
								Rectangle r = GetAppointmentTextRectangle(ap, true);
								Rectangle rect = new Rectangle(r.Right - 30, r.Top, 30, 10);
								g.FillRectangle(new SolidBrush(Color.LemonChiffon), rect);
								g.DrawRectangle(new Pen(LineColor), rect);
								Point [] p = new Point[3];
								p[0] = new Point(rect.Left+3, rect.Top+2);
								p[1] = new Point(rect.Right-3, rect.Top+2);
								p[2] = new Point(rect.Left+rect.Width/2 , rect.Bottom-2);
								g.FillPolygon(new SolidBrush(DayLineColor), p);  
								g.DrawPolygon(new Pen(LineColor), p);  
							}
							break;
						case DayOfWeek.Monday  : d1++; 
							if (d1<=7)
								paintAppointment(g, ap);
							else
							{
								Rectangle r = GetAppointmentTextRectangle(ap, true);
								Rectangle rect = new Rectangle(r.Right - 30, r.Top, 30, 10);
								g.FillRectangle(new SolidBrush(Color.LemonChiffon), rect);
								g.DrawRectangle(new Pen(LineColor), rect);
								Point [] p = new Point[3];
								p[0] = new Point(rect.Left+3, rect.Top+2);
								p[1] = new Point(rect.Right-3, rect.Top+2);
								p[2] = new Point(rect.Left+rect.Width/2 , rect.Bottom-2);
								g.FillPolygon(new SolidBrush(DayLineColor), p);  
								g.DrawPolygon(new Pen(LineColor), p);  
							}
							break;
						case DayOfWeek.Tuesday  : d2++; 
							if (d2<=7)
								paintAppointment(g, ap);
							else
							{
								Rectangle r = GetAppointmentTextRectangle(ap, true);
								Rectangle rect = new Rectangle(r.Right - 30, r.Top, 30, 10);
								g.FillRectangle(new SolidBrush(Color.LemonChiffon), rect);
								g.DrawRectangle(new Pen(LineColor), rect);
								Point [] p = new Point[3];
								p[0] = new Point(rect.Left+3, rect.Top+2);
								p[1] = new Point(rect.Right-3, rect.Top+2);
								p[2] = new Point(rect.Left+rect.Width/2 , rect.Bottom-2);
								g.FillPolygon(new SolidBrush(DayLineColor), p);  
								g.DrawPolygon(new Pen(LineColor), p);  
							}
							break;
						case DayOfWeek.Wednesday  : d3++; 
							if (d3<=7)
								paintAppointment(g, ap);
							else
							{
								Rectangle r = GetAppointmentTextRectangle(ap, true);
								Rectangle rect = new Rectangle(r.Right - 30, r.Top, 30, 10);
								g.FillRectangle(new SolidBrush(Color.LemonChiffon), rect);
								g.DrawRectangle(new Pen(LineColor), rect);
								Point [] p = new Point[3];
								p[0] = new Point(rect.Left+3, rect.Top+2);
								p[1] = new Point(rect.Right-3, rect.Top+2);
								p[2] = new Point(rect.Left+rect.Width/2 , rect.Bottom-2);
								g.FillPolygon(new SolidBrush(DayLineColor), p);  
								g.DrawPolygon(new Pen(LineColor), p);  
							}
							break;
						case DayOfWeek.Thursday  : d4++; 
							if (d4<=7)
								paintAppointment(g, ap);
							else
							{
								Rectangle r = GetAppointmentTextRectangle(ap, true);
								Rectangle rect = new Rectangle(r.Right - 30, r.Top, 30, 10);
								g.FillRectangle(new SolidBrush(Color.LemonChiffon), rect);
								g.DrawRectangle(new Pen(LineColor), rect);
								Point [] p = new Point[3];
								p[0] = new Point(rect.Left+3, rect.Top+2);
								p[1] = new Point(rect.Right-3, rect.Top+2);
								p[2] = new Point(rect.Left+rect.Width/2 , rect.Bottom-2);
								g.FillPolygon(new SolidBrush(DayLineColor), p);  
								g.DrawPolygon(new Pen(LineColor), p);  
							}
							break;
						case DayOfWeek.Friday  : d5++; 
							if (d5<=7)
								paintAppointment(g, ap);
							else
							{
								Rectangle r = GetAppointmentTextRectangle(ap, true);
								Rectangle rect = new Rectangle(r.Right - 30, r.Top, 30, 10);
								g.FillRectangle(new SolidBrush(Color.LemonChiffon), rect);
								g.DrawRectangle(new Pen(LineColor), rect);
								Point [] p = new Point[3];
								p[0] = new Point(rect.Left+3, rect.Top+2);
								p[1] = new Point(rect.Right-3, rect.Top+2);
								p[2] = new Point(rect.Left+rect.Width/2 , rect.Bottom-2);
								g.FillPolygon(new SolidBrush(DayLineColor), p);  
								g.DrawPolygon(new Pen(LineColor), p);  
							}
							break;
						case DayOfWeek.Saturday  : d6++; 
							if (d6<=7)
								paintAppointment(g, ap);
							else
							{
								Rectangle r = GetAppointmentTextRectangle(ap, true);
								Rectangle rect = new Rectangle(r.Right - 30, r.Top, 30, 10);
								g.FillRectangle(new SolidBrush(Color.LemonChiffon), rect);
								g.DrawRectangle(new Pen(LineColor), rect);
								Point [] p = new Point[3];
								p[0] = new Point(rect.Left+3, rect.Top+2);
								p[1] = new Point(rect.Right-3, rect.Top+2);
								p[2] = new Point(rect.Left+rect.Width/2 , rect.Bottom-2);
								g.FillPolygon(new SolidBrush(DayLineColor), p);  
								g.DrawPolygon(new Pen(LineColor), p);  
							}
							break;
					}
				}
			}
			*/
			base.OnPaint(pe);
		}

		#endregion

		#region Keyboard

		internal override bool processKey(KeyEventArgs e) {
			int selRes;
			TimeSpan ts;
			switch (e.KeyData) {
				case Keys.Down:					
				case Keys.Shift | Keys.Down:					
					moveCursor(e.Shift, OneDay, OneDay, SelectedResource); 
					return true;
				case Keys.Up:					
				case Keys.Shift | Keys.Up:
					moveCursor(e.Shift, -OneDay, OneDay, SelectedResource); 
					return true;
				case Keys.Right:				
				case Keys.Shift | Keys.Right:
					if (addColumnIndex(1, out selRes, out ts))
						moveCursor(e.Shift, ts, OneDay, selRes); 
					return true;
				case Keys.Left:					
				case Keys.Shift | Keys.Left:					
					if (addColumnIndex(-1, out selRes, out ts))
						moveCursor(e.Shift, -ts, OneDay, selRes); 
					return true;
				case Keys.Home:					
				case Keys.Shift | Keys.Home:					
				case Keys.Control | Keys.Home:	
				case Keys.Shift | Keys.Control | Keys.Home:	
					moveCursor(e.Shift, CalendarTools.GetMondayDate(SelectedDateBegin), OneDay, SelectedResource); 
					return true;
				case Keys.End:					
				case Keys.Shift | Keys.End:					
				case Keys.Control | Keys.End:	
				case Keys.Shift | Keys.Control | Keys.End:	
					moveCursor(e.Shift, CalendarTools.GetMondayDate(SelectedDateBegin).AddDays(6), OneDay, SelectedResource); 
					return true;
			}
			return false;
		}

		bool addColumnIndex(int delta, out int resourceIndex, out TimeSpan ts) {
			resourceIndex = SelectedResource;
			switch (SelectedDateBegin.DayOfWeek) {
				case DayOfWeek.Monday:
				case DayOfWeek.Tuesday:
				case DayOfWeek.Wednesday:
				case DayOfWeek.Sunday:
					if (delta>0)
						ts = new TimeSpan(3, 0, 0, 0);
					else
						ts = new TimeSpan(4, 0, 0, 0);
					break;
				default:
					if (delta>0)
						ts = new TimeSpan(4, 0, 0, 0);
					else
						ts = new TimeSpan(3, 0, 0, 0);
					break;
			}
			if (!isResourceBar) return true;

			resourceIndex += delta;
			if (resourceIndex>=Resources.Count) {
				resourceIndex = 0;
				return true;
			}
			else if (resourceIndex<0) {
				resourceIndex = Resources.Count-1;
				return true;
			}
			SelectedResource = resourceIndex;
			return false;
		}

		#endregion
	}

#endif
}
