#region Copyright (c) 2000-2004 ComponentGo!
/*
*********************************************************************
*                                                                   *
*       ComponentGo! Component Library                              *
*                                                                   *
*       Copyright (c) 2000-2004 ComponentGo!                        *
*       ALL RIGHTS RESERVED                                         *
*                                                                   *
*********************************************************************
*/
#endregion

using System;
using System.Windows.Forms;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Collections;
using System.Diagnostics;

namespace ComponentGo.Forms
{
	/// <summary>
	/// This class facilitates specifying MessageBox parameters. 
	/// </summary>
	/// <remarks>
	/// The class can be used in design mode, where all message parameters 
	/// can be set. Still in design mode you can see how a message box is 
	/// going to look like, right-clicking a MsgBox component and selecting 
	/// Show message box.<para/>
	/// A text shown in a window is specified by setting a <see cref="Text"/> 
	/// property. Other window parameters are specified by setting 
	/// <see cref="Caption"/>, <see cref="Icon"/>, <see cref="Buttons"/> 
	/// and <see cref="DefaultButton"/> properties. A window is displayed 
	/// at runtime by a call to <see cref="ShowDialog"/>. It can be passed 
	/// parameters used by a <see cref="String.Format"/> method.<para/>
	/// Message box can write information about being displayed and an answer 
	/// given to a system Event Log. This is enable by setting a 
	/// <see cref="LogSourceName"/> property.
	/// </remarks>
	/// <example>
	/// An example shows a message box displaying a current date.
	/// <code>
	/// MsgBox mb = new MsgBox();
	/// mb.Caption = "Information - ComponentGo!";
	/// mb.Icon = MsgBoxIcon.Exclamation;
	/// mb.Text = "Today is {0}.";
	/// mb.ShowDialog(DateTime.Today);
	/// </code>
	/// </example>
	[Designer(typeof(MsgBox.MyDesigner))]
	public class MsgBox : System.ComponentModel.Component
	{
		#region Editor

		internal class MyDesigner : IDesigner {

			IComponent component;

			public IComponent Component {
				get { return component; }
			}

			public void Initialize(IComponent component) {
				this.component = component;
			}

			public DesignerVerbCollection Verbs {
				get {
					return new DesignerVerbCollection(new DesignerVerb[] {
							new DesignerVerb("&Show message box", new EventHandler(showMessageBox))
						});
				}
			}

			void showMessageBox(object sender, EventArgs args) {
				MsgBox mb = Component as MsgBox;
				if (mb!=null)
					mb.ShowDialog();
			}

			public void DoDefaultAction() {
			}

			public void Dispose() {
			}
		}

		#endregion

		#region Properties

		string text = "";

		/// <summary>
		/// The text to display in the message box.
		/// </summary>
		/// <remarks>
		/// It can contain parameters for <see cref="String.Format"/> in a form 
		/// of for example <b>{0}</b>. <see cref="ShowDialog"/> method parameters 
		/// are then used to format a message. 
		/// <seealso cref="Lines"/>
		/// </remarks>
		[Bindable(true)]
		[Category("Appearance")]
		[Description("The text to display in the message box. It can contain format parameters like {0}.")]
		[DefaultValue("")]
		public string Text {
			get { return text; }
			set { text = value; }
		}

		/// <summary>
		/// A text (an array of lines) to be displayed in a message box.
		/// </summary>
		/// <remarks>
		/// This property contains the same value as a <see cref="Text"/> 
		/// property, but copied into an array of text lines, facilitating 
		/// editing them in a  design mode.<para/> It can contain parameters 
		/// for <see cref="String.Format"/> in a form for example <b>{0}</b>. 
		/// <see cref="ShowDialog"/> method parameters are then used to format 
		/// a message. 
		/// <seealso cref="Text"/>
		/// </remarks>
		[Category("Appearance")]
		[Description("The text to display in the message box. It can contain format parameters like {0}.")]
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public string [] Lines {
			get { 
				return Text.Split('\n'); 
			}
			set { 
				Text = string.Join("\n", value);
			}
		}

		string caption = "";

		/// <summary>
		/// The text to display in the title bar of the message box. 
		/// </summary>
		/// <remarks>
		/// It can contain parameters for <see cref="String.Format"/> 
		/// in a form of for example <b>{0}</b>. <see cref="ShowDialog"/> 
		/// method parameters are then used to format a message.
		/// </remarks>
		[DefaultValue("")]
		[Bindable(true)]
		[Category("Appearance")]
		[Description("The text to display in the title bar of the message box. It can contain format parameters like {0}.")]
		public string Caption {
			get { return caption; }
			set { caption = value; }
		}

		MessageBoxButtons buttons = MessageBoxButtons.OK;

		/// <summary>
		/// Specifies which buttons to display in the message box.
		/// </summary>
		[DefaultValue(MessageBoxButtons.OK)]
		[Category("Behavior")]
		[Description("Specifies which buttons to display in the message box.")]
		public MessageBoxButtons Buttons {
			get { return buttons; }
			set { buttons = value; }
		}

		MsgBoxIcon icon = MsgBoxIcon.None;

		/// <summary>
		/// Specifies which icon to display in the message box.
		/// </summary>
		[DefaultValue(MsgBoxIcon.None)]
		[Category("Appearance")]
		[Description("Specifies which icon to display in the message box.")]
		public MsgBoxIcon Icon {
			get { return icon; }
			set { icon = value; }
		}

		MessageBoxDefaultButton defaultButton = MessageBoxDefaultButton.Button1;

		/// <summary>
		/// Specifies the default button for the message box.
		/// </summary>
		[DefaultValue(MessageBoxDefaultButton.Button1)]
		[Category("Behavior")]
		[Description("Specifies the default button for the message box.")]
		public MessageBoxDefaultButton DefaultButton {
			get { return defaultButton; }
			set { defaultButton = value; }
		}

		MessageBoxOptions options = (MessageBoxOptions)0;

		/// <summary>
		/// The message box is displayed on the active desktop.
		/// </summary>
		/// <remarks>
		/// This property is the same as <see cref="ServiceNotification"/> except that 
		/// the system displays the message box only on the default desktop of 
		/// the interactive window station.
		/// </remarks>
		[DefaultValue(false)]
		[Category("Behavior")]
		[Description("The message box is displayed on the active desktop.")]
		public bool DefaultDesktopOnly {
			get { return (options & MessageBoxOptions.DefaultDesktopOnly)!=0; }
			set { 
				if (value)
					options |= MessageBoxOptions.DefaultDesktopOnly;
				else
					options &= ~MessageBoxOptions.DefaultDesktopOnly;
			}
		}

		/// <summary>
		/// The message box text is right-aligned.
		/// </summary>
		[DefaultValue(false)]
		[Category("Appearance")]
		[Description("The message box text is right-aligned.")]
		public bool RightAlign {
			get { return (options & MessageBoxOptions.RightAlign)!=0; }
			set { 
				if (value)
					options |= MessageBoxOptions.RightAlign;
				else
					options &= ~MessageBoxOptions.RightAlign;
			}
		}

		/// <summary>
		/// Specifies that the message box text is displayed with right to left reading order.
		/// </summary>
		[DefaultValue(false)]
		[Category("Appearance")]
		[Description("Specifies that the message box text is displayed with right to left reading order.")]
		public bool RtlReading {
			get { return (options & MessageBoxOptions.RtlReading)!=0; }
			set { 
				if (value)
					options |= MessageBoxOptions.RtlReading;
				else
					options &= ~MessageBoxOptions.RtlReading;
			}
		}

		/// <summary>
		/// The message box is displayed on the active desktop. 
		/// </summary>
		/// <remarks>
		/// The caller is a service notifying the user of an event. The function displays 
		/// a message box on the current active desktop, even if there is no user 
		/// logged on to the computer.
		/// </remarks>
		[DefaultValue(false)]
		[Category("Behavior")]
		[Description("The message box is displayed on the active desktop.")]
		public bool ServiceNotification {
			get { return (options & MessageBoxOptions.ServiceNotification)!=0; }
			set { 
				if (value)
					options |= MessageBoxOptions.ServiceNotification;
				else
					options &= ~MessageBoxOptions.ServiceNotification;
			}
		}

		#endregion

		#region Event log

		string logSourceName = "";

		/// <summary>
		/// Name of an event log source, the messages about anwers gien will be written to.
		/// </summary>
		/// <remarks>
		/// Displaying a message can be written to a system event log. We can record 
		/// important information used to trace a system behaviour. <para/> 
		/// Setting this property to a value different than an empty string, enables 
		/// writing a message to an event log. A default empty string or a <b>null</b> 
		/// disables writing to an event log.
		/// </remarks>
		[Category("Behavior")]
		[Description("Name of an event log source, the messages about answers gien will be written to.")]
		[DefaultValue("")]
		public string LogSourceName {
			get { return logSourceName; }
			set { logSourceName = value; }
		}

		void writeEventLog(string msg, DialogResult result) {
//			try {
			using (EventLog el = new EventLog("", ".", LogSourceName)) {
				EventLogEntryType type = EventLogEntryType.Information;
				switch (Icon) {
					case MsgBoxIcon.Stop: type = EventLogEntryType.Error; break;
					case MsgBoxIcon.Exclamation: type = EventLogEntryType.Warning; break;
				}

				el.WriteEntry(msg+" "+result, type);
			}
//			}
//			catch {
//			}
		}

		#endregion

		#region ShowDialog

		static readonly object [] EmptyArray = new object[0];

		/// <summary>
		/// Displays a message box.
		/// </summary>
		/// <returns>One of the <see cref="DialogResult"/> values.</returns>
		public DialogResult ShowDialog() {
			return show(null, null, EmptyArray);
		}

		/// <summary>
		/// Displays a message box in front of the specified object.
		/// </summary>
		/// <param name="owner">The <see cref="IWin32Window"/> the message box will display in front of.</param>
		/// <returns>One of the <see cref="DialogResult"/> values.</returns>
		public DialogResult ShowDialog(IWin32Window owner) {
			return show(owner, null, EmptyArray);
		}


		/// <summary>
		/// Displays a message box with the specified arguments.
		/// </summary>
		/// <param name="args">An <see cref="Object"/> array containing zero 
		///		or more objects to format text and caption of message box.</param>
		/// <returns>One of the <see cref="DialogResult"/> values.</returns>
		public DialogResult ShowDialog(params object [] args) {
			return show(null, null, args);
		}

		/// <summary>
		/// Displays a message box in front of the specified object and with the specified arguments.
		/// </summary>
		/// <param name="owner">The <see cref="IWin32Window"/> the message box will display in front of.</param>
		/// <param name="args">An <see cref="Object"/> array containing zero 
		///		or more objects to format text and caption of message box.</param>
		/// <returns>One of the <see cref="DialogResult"/> values.</returns>
		public DialogResult ShowDialog(IWin32Window owner, params object [] args) {
			return show(owner, null, args);
		}

		/// <summary>
		/// Displays a message box with the specified arguments.
		/// </summary>
		/// <param name="provider">An <see cref="IFormatProvider"/> that supplies culture-specific formatting information.</param>
		/// <param name="args">An <see cref="Object"/> array containing zero 
		///		or more objects to format text and caption of message box.</param>
		/// <returns>One of the <see cref="DialogResult"/> values.</returns>
		public DialogResult ShowDialog(IFormatProvider provider, params object [] args) {
			return show(null, null, args);
		}

		/// <summary>
		/// Displays a message box in front of the specified object and with the specified arguments.
		/// </summary>
		/// <param name="owner">The <see cref="IWin32Window"/> the message box will display in front of.</param>
		/// <param name="provider">An <see cref="IFormatProvider"/> that supplies culture-specific formatting information.</param>
		/// <param name="args">An <see cref="Object"/> array containing zero 
		///		or more objects to format text and caption of message box.</param>
		/// <returns>One of the <see cref="DialogResult"/> values.</returns>
		public DialogResult ShowDialog(IWin32Window owner, IFormatProvider provider, params object [] args) {
			return show(owner, null, args);
		}

		#endregion

		#region Events

		static readonly object keyOK = new object();
		static readonly object keyAbort = new object();
		static readonly object keyCancel = new object();
		static readonly object keyIgnore = new object();
		static readonly object keyNo = new object();
		static readonly object keyRetry = new object();
		static readonly object keyYes = new object();

		/// <summary>
		/// Event called when an OK button on a message box is pressed.
		/// </summary>
		[Category("Action")]
		[Description("Event called when an OK button on a message box is pressed.")]
		public event EventHandler OKClick {
			add { Events.AddHandler(keyOK, value); }
			remove { Events.RemoveHandler(keyOK, value); }
		}

		/// <summary>
		/// Event called when an ABORT button on a message box is pressed.
		/// </summary>
		[Category("Action")]
		[Description("Event called when an ABORT button on a message box is pressed.")]
		public event EventHandler AbortClick {
			add { Events.AddHandler(keyAbort, value); }
			remove { Events.RemoveHandler(keyAbort, value); }
		}

		/// <summary>
		/// Event called when an CANCEL button on a message box is pressed.
		/// </summary>
		[Category("Action")]
		[Description("Event called when an CANCEL button on a message box is pressed.")]
		public event EventHandler CancelClick {
			add { Events.AddHandler(keyCancel, value); }
			remove { Events.RemoveHandler(keyCancel, value); }
		}

		/// <summary>
		/// Event called when an IGNORE button on a message box is pressed.
		/// </summary>
		[Category("Action")]
		[Description("Event called when an IGNoRE button on a message box is pressed.")]
		public event EventHandler IgnoreClick {
			add { Events.AddHandler(keyIgnore, value); }
			remove { Events.RemoveHandler(keyIgnore, value); }
		}

		/// <summary>
		/// Event called when an NO button on a message box is pressed.
		/// </summary>
		[Category("Action")]
		[Description("Event called when an NO button on a message box is pressed.")]
		public event EventHandler NoClick {
			add { Events.AddHandler(keyNo, value); }
			remove { Events.RemoveHandler(keyNo, value); }
		}

		/// <summary>
		/// Event called when an RETRY button on a message box is pressed.
		/// </summary>
		[Category("Action")]
		[Description("Event called when an RETRY button on a message box is pressed.")]
		public event EventHandler RetryClick {
			add { Events.AddHandler(keyRetry, value); }
			remove { Events.RemoveHandler(keyRetry, value); }
		}

		/// <summary>
		/// Event called when an YES button on a message box is pressed.
		/// </summary>
		[Category("Action")]
		[Description("Event called when an YES button on a message box is pressed.")]
		public event EventHandler YesClick {
			add { Events.AddHandler(keyYes, value); }
			remove { Events.RemoveHandler(keyYes, value); }
		}

		#endregion

		DialogResult show(IWin32Window owner, IFormatProvider provider, params object [] args) {
			string text = Text;
			string caption = Caption;
			if (args.Length!=0) {
				text = string.Format(provider, text, args);
				caption = string.Format(provider, caption, args);
			}

			DialogResult result = MessageBox.Show(owner, text, caption, Buttons, 
				(MessageBoxIcon)Icon, DefaultButton, options);

			if (LogSourceName!=null && LogSourceName!="" && !DesignMode)
				writeEventLog(text, result);

			object key = null;
			switch (result) {
				case DialogResult.OK: key = keyOK; break;
				case DialogResult.Abort: key = keyAbort; break;
				case DialogResult.Cancel: key = keyCancel; break;
				case DialogResult.Ignore: key = keyIgnore; break;
				case DialogResult.No: key = keyNo; break;
				case DialogResult.Retry: key = keyRetry; break;
				case DialogResult.Yes: key = keyYes; break;
			}
			if (key!=null) {
				EventHandler handler = (EventHandler)Events[key];
				if (handler!=null)
					handler(this, EventArgs.Empty);
			}

			return result;
		}
	}

	#region MsgBoxIcon

	/// <summary>
	/// Specifies constants defining which information to display.
	/// </summary>
	/// <remarks>
	/// A subset of distinct <see cref="MessageBoxIcon"/> values.<para/>
	/// This enumeration is used by the <see cref="MsgBox"/> class. The description 
	/// of each member of this enumeration contains a typical representation 
	/// of the symbol. The actual graphic displayed is a function of the operating 
	/// system constants. In current implementations there are four unique symbols 
	/// with multiple values assigned to them.
	/// </remarks>
	public enum MsgBoxIcon {
		/// <summary>
		/// The message box contain no symbols.
		/// </summary>
		None = (int)MessageBoxIcon.None,
		/// <summary>
		/// The message box contains a symbol consisting of white X in a circle 
		/// with a red background.
		/// </summary>
		Stop = (int)MessageBoxIcon.Stop,
		/// <summary>
		/// The message box contains a symbol consisting of a question mark in a circle.
		/// </summary>
		Question = (int)MessageBoxIcon.Question,
		/// <summary>
		/// The message box contains a symbol consisting of an exclamation point 
		/// in a triangle with a yellow background.
		/// </summary>
		Exclamation = (int)MessageBoxIcon.Exclamation,
		/// <summary>
		/// The message box contains a symbol consisting of a lowercase letter i in a circle.
		/// </summary>
		Asterisk = (int)MessageBoxIcon.Asterisk,
	}

	#endregion
}
