Imports System.ComponentModel
Imports System.Diagnostics
Imports System.IO
Imports System.Threading
Imports System.Windows.Forms

Imports Phoenix.DataAccess
Imports Phoenix.Events
Imports Phoenix.Forms
Imports Phoenix.Forms.View
Imports Phoenix.Framework
Imports Phoenix.Helpers
Imports Phoenix.Preferences
Imports Phoenix.Preferences.Store
Imports Phoenix.Security
Imports Phoenix.Till

Module PhoenixApplication

#Region " Trace Configuration "
    Public ReadOnly AppTraceSwicth As TraceSwitch = New TraceSwitch("General", "Entire Application")

    Public ReadOnly Property AppTrace() As Boolean
        Get
            Return AppTraceSwicth.TraceVerbose
        End Get
    End Property

    Public ReadOnly Property AppInfo() As Boolean
        Get
            Return AppTraceSwicth.TraceInfo
        End Get
    End Property

    Public ReadOnly Property AppWarn() As Boolean
        Get
            Return AppTraceSwicth.TraceWarning
        End Get
    End Property

    Public ReadOnly Property AppError() As Boolean
        Get
            Return AppTraceSwicth.TraceError
        End Get
    End Property
#End Region

#Region " Startup Code "
    ''' <summary>
    ''' The application startup entry.
    ''' </summary>
    ''' <param name="Arguments">The program's startup arguments.</param>
    <STAThread()> _
    Public Sub Main(ByVal Args() As String)
        Console.WriteLine("Phoenix iTill System (C) 2005-2006 by thEngine Technology Services.")
        Trace.Listeners.Add(New TextWriterTraceListener(Console.Out))
        Try
            Dim logName As String

            logName = String.Format("Phoenix{0:yyyyMMdd}_{0:HHmm}.log", DateTime.Now)

            Trace.Listeners.Add(New PhoenixTraceListener(logName))
        Catch ex As Exception
            Trace.WriteLine(ex)
        End Try

        Thread.CurrentThread.CurrentCulture = New System.Globalization.CultureInfo("en-GB")
        Thread.CurrentThread.CurrentUICulture = New System.Globalization.CultureInfo("en-GB")

        'find another application instance running
        Dim createdNew As Boolean
        Dim m As Mutex = New Mutex(True, "PhoenixRunningInstanceDetectionMutex", createdNew)
        Try
            If createdNew Then
                ' Run new application instance
                mAppEventListener = New GlobalEventListener
                Application.AddMessageFilter(mAppEventListener)
                MessageFilter.Register()
                Try
                    Run()
                Finally
                    MessageFilter.Revoke()
                    Application.RemoveMessageFilter(mAppEventListener)
                End Try
            Else
                ' Activate the previous application instance's main window
                Debug.WriteLineIf(AppWarn, "Another application instance is already running")
                WindowsActivator.ActivateWindow(Res.ApplicationTitle)
            End If
        Finally
            m.Close()
        End Try

    End Sub

    ''' <summary>
    ''' Runs the new application instance.
    ''' </summary>
    Private Sub Run()
        Trace.WriteLineIf(AppInfo, String.Format("{0} , .NET Runtime {1}", _
            EnvironmentHelper.GetOsNameWithVersion(Environment.OSVersion), _
            Environment.Version.ToString()))
        Trace.WriteLineIf(AppInfo, Res.ApplicationTitle)

        'Enable visual styles
        Application.EnableVisualStyles()
        Application.DoEvents()

        ' Set global exception hook
        AddHandler Application.ThreadException, AddressOf OnInternalError

        'Reset user identity
        SecurityHelper.ResetUser()

        ShowSplash()

        LoadConfig()
        InitializePreferensesStore()
        InitializeDataLayer()

        SplashMessage = Res.ProgressRunningApplication
        Try
            mMainForm = New MainForm
            If AppConfig.UserModeEnabled Then
                mMainForm.CreateView(GetType(LoginView))
            Else
                Dim e As DataSetData.Employee = Nothing
                Select Case AppConfig.DefaultSecurityLevel
                    Case SecurityRole.Administrator
                        e = AppDataLayer.Employees.FindByEmployeeID(1)
                    Case SecurityRole.Manager
                        e = AppDataLayer.Employees.FindByEmployeeID(2)
                    Case SecurityRole.[Operator]
                        e = AppDataLayer.Employees.FindByEmployeeID(3)
                End Select

                If e Is Nothing Then
                    mMainForm.CreateView(GetType(LoginView))
                Else
                    AppLogin(e)
                End If
            End If
            Application.Run(mMainForm)
        Catch ex As Exception
            Trace.WriteIf(AppError, ex)
        End Try
        SaveConfiguration()
    End Sub

#End Region

#Region " Splash "
    Private mSplashThread As Thread
    Private mSplash As SplashForm

    Public Sub ShowSplash()
        If Not IsNothing(mSplash) Then Exit Sub

        mSplashThread = New Thread(New ThreadStart(AddressOf SplashThread))
        mSplashThread.IsBackground = True
        mSplashThread.ApartmentState = ApartmentState.STA
        mSplashThread.Start()

        Do While IsNothing(mSplash)
            Thread.Sleep(New TimeSpan(100))
        Loop
    End Sub

    Public Sub HideSplash()
        If IsNothing(mSplash) Then Exit Sub
        If IsNothing(mSplashThread) Then Exit Sub

        Try
            If mSplash.InvokeRequired Then
                mSplash.Invoke(New MethodInvoker(AddressOf mSplash.Close))
            Else
                mSplash.Close()
            End If
        Catch
        End Try

        mSplashThread = Nothing
        mSplash = Nothing
    End Sub

    Public WriteOnly Property SplashMessage() As String
        Set(ByVal Value As String)
            If IsNothing(mSplash) Then Exit Property
            mSplash.Message = Value
            Trace.WriteLineIf(AppInfo, Value)
        End Set
    End Property

    Private Sub SplashThread()
        mSplash = New SplashForm
        Application.Run(mSplash)
    End Sub
#End Region

#Region " Implementaion Routines "

    ''' <summary>
    ''' Establishes connection with database.
    ''' </summary>
    Private Sub InitializeDataLayer()
        SplashMessage = Res.ProgressInitDataLayer

        Dim conn As String = System.Configuration.ConfigurationSettings.AppSettings.Item("Phoenix.Connection")
        If conn Is Nothing Then
            conn = String.Format("server={0}\Phoenix;database=Phoenix; User Id=sa; Pwd=gfhjkmCF", Environment.MachineName)
        End If
        mAppDataLayer = New DataLayer(conn)
    End Sub


    ''' <summary>
    ''' Loads configuration settings.
    ''' </summary>
    Private Sub LoadConfig()
        SplashMessage = Res.ProgressLoadConfig

        Dim stream As System.IO.Stream
        Try
            stream = System.IO.File.Open(Path.Combine(EnvironmentHelper.ApplicationPath, "Phoenix.config"), System.IO.FileMode.OpenOrCreate)
            Dim formatter As New System.Runtime.Serialization.Formatters.Binary.BinaryFormatter

            mAppConfig = CType(formatter.Deserialize(stream), PhoenixConfiguration)
        Catch ex As Exception
            Debug.WriteIf(AppTrace, "Resetting config")
            AppConfig.LoadDefaults()
        Finally
            stream.Close()
        End Try
    End Sub


    ''' <summary>
    ''' Saves configuration settings.
    ''' </summary>
    Private Sub SaveConfiguration()
        Dim stream As System.IO.Stream
        Try
            'TODO:    
            stream = System.IO.File.Open(Path.Combine(EnvironmentHelper.ApplicationPath, "Phoenix.config"), IO.FileMode.Create, FileAccess.Write)
            Dim formatter As New System.Runtime.Serialization.Formatters.Binary.BinaryFormatter

            formatter.Serialize(stream, AppConfig)

        Catch ex As Exception

        Finally
            stream.Close()
        End Try
    End Sub

    ''' <summary>
    ''' Initializes the prefereneces root node
    ''' </summary>
    Private Sub InitializePreferensesStore()
        mAppPreferences = PreferencesFactory.GetSystemRootNode(WindowsFileSystemStore.Instance)
    End Sub


    ''' <summary>
    ''' This method is invoded then exception is thrown out of the application boundaries
    ''' </summary>
    ''' <param name="sender">The component which throws exception</param>
    ''' <param name="e">The thrown exception.</param>
    Private Sub OnInternalError(ByVal sender As Object, ByVal e As ThreadExceptionEventArgs)
        HideSplash()

        Trace.WriteLineIf(AppError, "FATAL ERROR:")
        Trace.WriteLineIf(AppError, e.Exception)

#If DEBUG Then
        Dim dlg As New UnhandledErrorForm
        dlg.Description = System.Reflection.Assembly.GetEntryAssembly().GetName.Version.ToString() & vbCrLf & e.Exception.ToString
        If dlg.ShowDialog() = DialogResult.OK Then
            Try
                System.Diagnostics.Process.Start("mailto:phoenix@thengine.com?body=" & System.Web.HttpUtility.UrlEncode(System.Reflection.Assembly.GetEntryAssembly().GetName.Version.ToString() & vbCrLf & e.Exception.ToString) & "&subject=Phoenix%20Unhandled%20Error")
            Catch ex As Exception
                System.Diagnostics.Process.Start("mailto:phoenix@thengine.com?subject=Phoenix%20Unhandled%20Error")
            End Try
        End If
#Else
        If MessageBox.Show("The critical error occured. It is recommended ro restart the application." & vbCrLf & _
            "Do you want to exit from the application now?", "Critical Error", MessageBoxButtons.YesNo, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1) = DialogResult.Yes Then
            Application.Exit()
        End If
#End If

    End Sub

#End Region

#Region " Globals "
    Public Const AppIconColumnPadding As Integer = 6

    Private mAppEventListener As GlobalEventListener
    Public ReadOnly Property AppEventListener() As GlobalEventListener
        Get
            Return mAppEventListener
        End Get
    End Property

    Private mMainForm As MainForm
    Public ReadOnly Property AppMainWindow() As MainForm
        Get
            Return mMainForm
        End Get
    End Property

    ''' <summary>
    ''' Stores the data gateway.
    ''' </summary>
    Public WithEvents mAppDataLayer As DataLayer
    Public ReadOnly Property AppDataLayer() As DataLayer
        Get
            Debug.Assert(Not mAppDataLayer Is Nothing)
            Return mAppDataLayer
        End Get
    End Property

    Private mAppConfig As PhoenixConfiguration
    Public ReadOnly Property AppConfig() As PhoenixConfiguration
        Get
            If mAppConfig Is Nothing Then
                mAppConfig = New PhoenixConfiguration
                mAppConfig.LoadDefaults()
            End If
            Return mAppConfig
        End Get
    End Property



    Private mAppCursor As New CursorHelper
    Public ReadOnly Property AppCursor() As CursorHelper
        Get
            If mAppCursor Is Nothing Then mAppCursor = New CursorHelper
            Return mAppCursor
        End Get
    End Property

    ''' <summary>
    ''' The root node of preferences store.
    ''' </summary>
    Private mAppPreferences As IPreferencesNode
    Public ReadOnly Property AppPreferences() As IPreferencesNode
        Get
            Return mAppPreferences
        End Get
    End Property

    Public ReadOnly Property AppUserPreferences() As IPreferencesNode
        Get
            Return mAppPreferences
        End Get
    End Property


    Private mAppSharedComponents As New PhoenixSharedComponents
    Public ReadOnly Property AppSharedComponents() As PhoenixSharedComponents
        Get
            Return mAppSharedComponents
        End Get
    End Property

    Public Function AppMessageBox(ByVal text As String, Optional ByVal buttons As MsgBoxStyle = MsgBoxStyle.DefaultButton1) As MsgBoxResult
        Return MsgBox(text, buttons)
    End Function

    Public Sub AppAbout()
        Dim frm As AboutForm = New AboutForm
        frm.ShowDialog()
    End Sub

    Public Event AppLogingIn(ByVal sender As Object, ByVal args As EventArgs)
    Public Event AppLogingOut(ByVal sender As Object, ByVal args As CancelEventArgs)

    Public Sub AppLogin(ByVal empl As DataAccess.DataSetData.Employee)
        SecurityHelper.AuthorizateUser(empl)
        mMainForm.CreateView(GetType(MainView))
    End Sub

    Public Sub AppLogout()
        Dim args As New CancelEventArgs(False)
        RaiseEvent AppLogingOut(Nothing, args)
        If args.Cancel Then Return

        SecurityHelper.ResetUser()
        mMainForm.CreateView(GetType(LoginView))
    End Sub

    Public Sub AppExit()
        AppLogout()
        mMainForm.Close()
    End Sub

#End Region

#Region " Notifications "
    Private mEventPool As ArrayList = New ArrayList
    Public Event AppNotification(ByVal sender As Object, ByVal e As PhoenixEventArgs)

    Public Sub AppRaiseEventDirect(ByVal sender As Object, ByVal e As PhoenixEventArgs)
        RaiseEvent AppNotification(sender, e)
    End Sub

    Public Sub AppRaiseEvent(ByVal sender As Object, ByVal e As PhoenixEventArgs)
        If AppDataLayer.Transaction Is Nothing Then
            AppRaiseEventDirect(sender, e)
        Else
            mEventPool.Add(New PhoenixEvent(sender, e))
        End If
    End Sub

    Public Sub PushEvents()
        For Each e As PhoenixEvent In mEventPool
            Try
                AppRaiseEventDirect(e.Sender, e.Args)
            Catch ex As Exception
                Debug.WriteLineIf(AppError, ex)
            End Try
        Next

        mEventPool.Clear()
    End Sub

    Public Sub ClearEvents()
        mEventPool.Clear()
    End Sub

    Private Sub DataLayer_TransactionCommitted(ByVal sender As Object, ByVal e As EventArgs) Handles mAppDataLayer.TransactionCommitted
        PushEvents()
    End Sub

    Private Sub DataLayer_TransactionRollbacked(ByVal sender As Object, ByVal e As EventArgs) Handles mAppDataLayer.TransactionRollbacked
        ClearEvents()
    End Sub
#End Region

#Region " Reports "
    Public Sub AppReport(ByVal templatePath As String, ByVal resultPath As String, ByVal prop As Object, ByVal ds As DataSet)
        Dim re As New ReportEngine.ReportEngine
        re.GenerateDocument(templatePath, resultPath, prop, ds)
    End Sub
#End Region

#Region " Cash Drawer & Recept Printer "
    Public Sub AppOpenTill()
        If Not AppConfig.OpenCashDrawer Then Return

        Dim man As New TillManager
        man.Open()
        Try
            AppOpenTill(man)
        Finally
            man.Close()
        End Try
    End Sub

    Public Sub AppOpenTillAndPrintReceipt(ByVal r As Receipt)
        If Not (AppConfig.OpenCashDrawer Or AppConfig.PrintReceipt) Then Return

        Dim man As New TillManager
        man.Open()
        Try
            AppOpenTill(man)
            AppPrintReceipt(man, r)
        Finally
            man.Close()
        End Try
    End Sub

    Private Sub AppOpenTill(ByVal man As TillManager)
        If Not AppConfig.OpenCashDrawer Then Return
        man.TillOpen()
    End Sub

    Private Sub AppPrintReceipt(ByVal man As TillManager, ByVal r As Receipt)
        If Not AppConfig.PrintReceipt Then Return
        man.PrintReceipt(r)
    End Sub
#End Region
End Module
